// Copyright (C) 1999-2007
// Smithsonian Astrophysical Observatory, Cambridge, MA, USA
// For conditions of distribution and use, see copyright notice in "copyright"

#include <iostream>
#include <sstream>
#include <iomanip>
using namespace std;

#include <tcl.h>
#include <Xlib.h>

// we need these routines because Tk's versions are too simplistic

extern "C" {

void XXWarpPointer(Display* display, Window src_w, Window dest_w,
		   int src_x, int src_y,
		   unsigned int src_width, unsigned int src_height,
		   int dest_x, int dest_y)
{
  if (dest_w != None)
    // move to absolute location
    XWarpPointer(display, src_w, dest_w, src_x, src_y, src_width, src_height,
		 dest_x, dest_y);
  else {
    cerr << "XXWarpPointer" << endl;
  }
}

void XXPutImage(Display* display, Drawable d, GC gc, XImage* image,
		int src_x, int src_y, int dest_x, int dest_y,
		unsigned int width, unsigned int height)
{
  if (image->bits_per_pixel <= 8) {
    /*
    XXPutImageBitMap(display, d, gc, image, src_x, src_y,
		     dest_x, dest_y, width, height);
    */
    cerr << "XXPutImage" << endl;
  }
  else
    TkPutImage(NULL, 0, display, d, gc, image, src_x, src_y,
	       dest_x, dest_y, width, height);
}

#if 0
void XXGetImageBitMap(Display *display, Drawable d, int x, int y,
		unsigned int width, unsigned int height,
		unsigned long plane_mask, int format)
{ 
  XImage *   imagePtr = NULL;
  Pixmap     pixmap = (Pixmap) NULL;
  Tk_Window  win = (Tk_Window) ((MacDrawable *) d)->winPtr;
  GC         gc;
  int        depth = 32;
  int        offset = 0;
  int        bitmap_pad = 32;
  int        bytes_per_line = 0;
    
  if (TkMacOSXGetDrawablePort(d)) {
    if (format == ZPixmap) {
      if (width > 0 && height > 0) {
	/* Tk_GetPixmap fails for zero width or height */
	pixmap = Tk_GetPixmap(display, d, width, height, depth);
      }
      if (win) {
	XGCValues values;
	gc = Tk_GetGC(win, 0, &values);
      } else {
	gc = XCreateGC(display, pixmap, 0, NULL);
      }
      if (pixmap) {
	XCopyArea(display, d, pixmap, gc, x, y, width, height, 0, 0);
      }
      imagePtr = XCreateImage(display, NULL, depth, format, offset,
			      (char*)TkMacOSXGetDrawablePort(pixmap), 
			      width, height, bitmap_pad, bytes_per_line);
      /* Track Pixmap underlying the XImage in the unused obdata field *
       * so that we can treat XImages coming from XGetImage specially. */
      imagePtr->obdata = (XPointer) pixmap;
      if (!win) {
	XFreeGC(display, gc);
      }
    } else {
      TkpDisplayWarning(
			"XGetImage: only ZPixmap types are implemented",
			"XGetImage Failure");
    }
  }
  return imagePtr;
}
#endif

XImage* XXGetImage(Display* display, Drawable drawable, int x, int y,
		   unsigned int width, unsigned int height,
		   unsigned long plane, int format)
{
  if (plane == AllPlanes && format == XYPixmap) {
    /*
    XXGetImageBitMap(display, drawable, x, y, width, height, plane, format);
    */
    cerr << "XXGetImage" << endl;
    return NULL;
  }
  else
    return XGetImage(display, drawable, x, y, width, height, plane, format);
}

Status XGetGCValues(Display* disp, GC gc, unsigned long mask, XGCValues* ret)
{
  if (mask & GCFunction) {ret->function = gc->function;}
  if (mask & GCPlaneMask) {ret->plane_mask = gc->plane_mask;}
  if (mask & GCForeground) {ret->foreground = gc->foreground;}
  if (mask & GCBackground) {ret->background = gc->background;}
  if (mask & GCLineWidth) {ret->line_width = gc->line_width;}	
  if (mask & GCLineStyle) {ret->line_style = gc->line_style;}
  if (mask & GCCapStyle) {ret->cap_style = gc->cap_style;}
  if (mask & GCJoinStyle) {ret->join_style = gc->join_style;}
  if (mask & GCFillStyle) {ret->fill_style = gc->fill_style;}
  if (mask & GCFillRule) {ret->fill_rule = gc->fill_rule;}
  if (mask & GCArcMode) {ret->arc_mode = gc->arc_mode;}
  if (mask & GCTile) {ret->tile = gc->tile;}
  if (mask & GCStipple) {ret->stipple = gc->stipple;}
  if (mask & GCTileStipXOrigin) {ret->ts_x_origin = gc->ts_x_origin;}
  if (mask & GCTileStipYOrigin) {ret->ts_y_origin = gc->ts_y_origin;}
  if (mask & GCFont) {ret->font = gc->font;}
  if (mask & GCSubwindowMode) {ret->subwindow_mode = gc->subwindow_mode;}
  if (mask & GCGraphicsExposures) 
    {ret->graphics_exposures = gc->graphics_exposures;}
  if (mask & GCClipXOrigin) {ret->clip_x_origin = gc->clip_x_origin;}
  if (mask & GCClipYOrigin) {ret->clip_y_origin = gc->clip_y_origin;}
  if (mask & GCClipMask) {;} // not implemented
  if (mask & GCDashOffset) {ret->dash_offset = gc->dash_offset;}
  if (mask & GCDashList) {ret->dashes = gc->dashes;}
}

void XCopyGC(Display* display, GC src, unsigned long mask, GC dest)
{
  if (mask & GCFunction) {dest->function = src->function;}
  if (mask & GCPlaneMask) {dest->plane_mask = src->plane_mask;}
  if (mask & GCForeground) {dest->foreground = src->foreground;}
  if (mask & GCBackground) {dest->background = src->background;}
  if (mask & GCLineWidth) {dest->line_width = src->line_width;}	
  if (mask & GCLineStyle) {dest->line_style = src->line_style;}
  if (mask & GCCapStyle) {dest->cap_style = src->cap_style;}
  if (mask & GCJoinStyle) {dest->join_style = src->join_style;}
  if (mask & GCFillStyle) {dest->fill_style = src->fill_style;}
  if (mask & GCFillRule) {dest->fill_rule = src->fill_rule;}
  if (mask & GCArcMode) {dest->arc_mode = src->arc_mode;}
  if (mask & GCTile) {dest->tile = src->tile;}
  if (mask & GCStipple) {dest->stipple = src->stipple;}
  if (mask & GCTileStipXOrigin) {dest->ts_x_origin = src->ts_x_origin;}
  if (mask & GCTileStipYOrigin) {dest->ts_y_origin = src->ts_y_origin;}
  if (mask & GCFont) {dest->font = src->font;}
  if (mask & GCSubwindowMode) {dest->subwindow_mode = src->subwindow_mode;}
  if (mask & GCGraphicsExposures) 
    {dest->graphics_exposures = src->graphics_exposures;}
  if (mask & GCClipXOrigin) {dest->clip_x_origin = src->clip_x_origin;}
  if (mask & GCClipYOrigin) {dest->clip_y_origin = src->clip_y_origin;}
  if (mask & GCDashOffset) {dest->dash_offset = src->dash_offset;}
  if (mask & GCDashList) {
    dest->dashes = src->dashes;
    (&(dest->dashes))[1] = 0;
  }

  if (mask & GCClipMask) {
    if (dest->clip_mask)
      free((void*)dest->clip_mask);
    /*
    dest->clip_mask = (Pixmap)ckalloc(sizeof(TkpClipMask));
    ((TkpClipMask*)dest->clip_mask)->type = TKP_CLIP_PIXMAP;
    ((TkpClipMask*)dest->clip_mask)->value.pixmap = src->clip_mask;
    */
  } else {
    dest->clip_mask = None;
  }
}

}
