// Copyright (C) 1999-2005
// Smithsonian Astrophysical Observatory, Cambridge, MA, USA
// For conditions of distribution and use, see copyright notice in "copyright"

%{
#define YYPARSE_PARAM fr
#define YYDEBUG 1

#define FR ((FrameBase*)fr)
#define FITSPTR (FR->findFits(1))

#include <math.h>
#include <string.h>
#include <iostream.h>

#include "framebase.h"
#include "frame.h"
#include "framergb.h"
#include "fitsimage.h"
#include "util.h"
#include "vector.h"
#include "coord.h"
#include "fvcontour.h"

extern int frlex(void);
extern void frerror(const char*);

int frsign;
int frsign2;

static CoordSystem currentCoord = IMAGE;
static SkyFrame currentSky = FK5;

static unsigned short defaultProps = Marker::SELECT | Marker::HIGHLITE |
		Marker::EDIT | Marker::MOVE | Marker::ROTATE | 
		Marker::DELETE | Marker::INCLUDE | Marker::SOURCE;
static unsigned short currentProps;
static char currentColor[16];
static int currentWidth;
static char currentFont[32];
static char currentText[80];

static List<Tag> taglist;
static List<CallBack> cblist;

static unsigned short propQMask;
static unsigned short propQValue;

static void setProps(unsigned short* props, unsigned short prop, int value);
%}

%union {
  char chr;
  char str[2048];
  void* ptr;
  int integer;
  double real;
  double vector[3];
}

%type <real> numeric
%type <integer> yesno

%type <real> angle
%type <real> optangle
%type <integer> precision
%type <real> sexagesimal
%type <vector> coord
%type <integer> coordSystem
%type <integer> wcsSystem
%type <integer> internalSystem
%type <integer> skyFrame
%type <integer> skyFormat

%type <integer> markerProperty
%type <integer> markerCallBack
%type <integer> markerFormat
%type <integer> markerLayer
%type <integer> pscolorspace
%type <integer> scaleType
%type <integer> shmType
%type <integer> colormapMode
%type <integer> contourmethod
%type <integer> incrLoad
%type <integer> layerType
%type <integer> gridType
%type <integer> compressType
%type <integer> fileNameType
%type <integer> smoothFunction

%token <real> REAL
%token <integer> INT
%token <str> STRING
%token <ptr> POINTER

%token <integer> HOUR
%token <integer> MINUTE
%token <real> SECOND
%token <real> DEGREE
%token <real> ARCMINUTE
%token <real> ARCSECOND
%token <real> RADIAN

%token <str> SEXSTR

%token ABOUT_
%token ALLOC_
%token ALLOCGZ_
%token ALIGN_
%token ALL_
%token AMPLIFIER_
%token ANALYSIS_
%token ANGLE_
%token ANNULUS_
%token APPEND_
%token ARCMIN_
%token ARCSEC_
%token AREA_
%token ARRAY_
%token ARROW_
%token AST_
%token AUTO_
%token AVERAGE_
%token B1950_
%token BACK_
%token BASE_
%token BBOX_
%token BEGIN_
%token BG_
%token BIN_
%token BITPIX_
%token BLOCK_
%token BOX_
%token BOXANNULUS_
%token BOXCAR_
%token BOXCIRCLE_
%token BPANDA_
%token BUFFER_
%token BW_
%token CALLBACK_
%token CANVAS_
%token CATALOG_
%token CENTER_
%token CHANNEL_
%token CIRCLE_
%token CIAO_
%token CLEAR_
%token CLICK_
%token CLIP_
%token COLORMAP_
%token COLOR_
%token COLORSCALE_
%token COLORSPACE_
%token COLS_
%token COMMAND_
%token COMPASS_
%token COMPOSITE_
%token CONTOUR_
%token COORDINATES_
%token COPY_
%token COUNT_
%token CPANDA_
%token CREATE_
%token CROSS_
%token CROSSHAIR_
%token CUBE_
%token CURSOR_
%token CUT_
%token CMYK_
%token DATA_
%token DATAMIN_
%token DATASEC_
%token DEBUG_
%token DEGREES_
%token DEFAULT_
%token DELETE_
%token DEPTH_
%token DETECTOR_
%token DIAMOND_
%token DIM_
%token DS9_
%token EDIT_
%token ECLIPTIC_
%token ELLIPSE_
%token ELLIPSEANNULUS_
%token END_
%token EPANDA_
%token EQUATORIAL_
%token ERASE_
%token EXPORT_
%token EXT_
%token FACTOR_
%token FALSE_
%token FAST_
%token FILE_
%token FILTER_
%token FIT_
%token FITS_
%token FITSY_
%token FIXED_
%token FK4_
%token FK5_
%token FONT_
%token FRONT_
%token FULL_
%token FUNCTION_
%token GALACTIC_
%token GAUSSIAN_
%token GET_
%token GLOBAL_
%token GRAPHICS_
%token GRAY_
%token GRID_
%token GZ_
%token HANDLE_
%token HAS_
%token HEADER_
%token HEIGHT_
%token HIDE_
%token HIGH_
%token HIGHLITE_
%token HISTEQU_
%token HISTOGRAM_
%token HMS_
%token HORIZONTAL_
%token ICRS_
%token ID_
%token IIS_
%token IMAGE_
%token INCLUDE_
%token INCR_
%token INFO_
%token INTEGER_
%token IRAF_
%token IRAFMIN_
%token J2000_
%token JPEG_
%token KEY_
%token KEYWORD_
%token LABEL_
%token LENGTH_
%token LEVEL_
%token LINE_
%token LINEAR_
%token LIST_
%token LOAD_
%token LOCAL_
%token LOG_
%token LOW_
%token MAGNIFIER_
%token MAP_
%token MARK_
%token MARKER_
%token MASK_
%token MASTER_
%token MESSAGE_
%token METHOD_
%token MINMAX_
%token MMAP_
%token MMAPINCR_
%token MOSAIC_
%token MODE_
%token MOTION_
%token MOVE_
%token NAME_
%token NAN_
%token NEW_
%token NEXT_
%token NO_
%token NONE_
%token NOW_
%token NUMBER_
%token OBJECT_
%token OFF_
%token ON_
%token ONLY_
%token OPTION_
%token ORIENT_
%token PAN_
%token PANNER_
%token PARAM_
%token PARSER_
%token PASTE_
%token PERF_
%token PHYSICAL_
%token PIXEL_
%token PNG_
%token POINT_
%token POLYGON_
%token POSTSCRIPT_
%token POW_
%token PPM_
%token PRESERVE_
%token PROJECTION_
%token PROPERTY_
%token PUBLICATION_
%token PROS_
%token RADIUS_
%token REGION_
%token REPLACE_
%token RESAMPLE_
%token RESET_
%token RESOLUTION_
%token RGB_
%token ROOT_
%token ROTATE_
%token RULER_
%token SAMPLE_
%token SAOIMAGE_
%token SAOTNG_
%token SAVE_
%token SCAN_
%token SCIENTIFIC_
%token SCOPE_
%token SEGMENT_
%token SELECT_
%token SET_
%token SEXAGESIMAL_
%token SHARED_
%token SHIFT_
%token SHMID_
%token SHOW_
%token SIZE_
%token SKY_
%token SLAVE_
%token SLICE_
%token SMMAP_
%token SMOOTH_
%token SOCKET_
%token SOCKETGZ_
%token SOURCE_
%token SQRT_
%token SQUARED_
%token SSHARED_
%token STATUS_
%token SUM_
%token SYSTEM_
%token TABLE_
%token TAG_
%token TEMPLATE_
%token TEXT_
%token THICK_
%token TIFF_
%token TO_
%token TOGGLE_
%token TOPHAT_
%token TRUE_
%token TYPE_
%token UNDO_
%token UNHIGHLITE_
%token UNLOAD_
%token UNSELECT_
%token UPDATE_
%token USER_
%token VALUE_
%token VAR_
%token VIEW_
%token VECTOR_
%token VERSION_
%token VERTEX_
%token VERTICAL_
%token WARP_
%token WCS_
%token WCSA_
%token WCSB_
%token WCSC_
%token WCSD_
%token WCSE_
%token WCSF_
%token WCSG_
%token WCSH_
%token WCSI_
%token WCSJ_
%token WCSK_
%token WCSL_
%token WCSM_
%token WCSN_
%token WCSO_
%token WCSP_
%token WCSQ_
%token WCSR_
%token WCSS_
%token WCST_
%token WCSU_
%token WCSV_
%token WCSW_
%token WCSX_
%token WCSY_
%token WCSZ_
%token WCS0_
%token WFPC2_
%token WIDTH_
%token XY_
%token YES_
%token ZMAX_
%token ZSCALE_
%token ZOOM_

%%

command : DEBUG_ debug

	| BIN_ bin
	| BG_ COLOR_ STRING {FR->bgColorCmd($3);}
	| CENTER_ {FR->centerCmd();}
	| CLEAR_ {FR->clearCmd();}
	| CLIP_ clip
	| COLORMAP_ colormap
	| COLORSCALE_ scaleType 
	  {FR->colorScaleCmd((FrScale::ColorScaleType)$2);}
	| CONTOUR_ contour
	| CROSSHAIR_ crosshair
	| DATASEC_ yesno {FR->DATASECCmd($2);}
	| EXPORT_ export
	| FITSY_ fitsy
	| GET_ get
	| GRID_ grid
	| HAS_ has
	| HIDE_ {FR->hideCmd();}
	| HIGHLITE_ yesno {FR->highliteCmd($2);}
	| IIS_ iis
	| LOAD_ load
	| MAGNIFIER_ magnifier
	| MARKER_ markerLayer marker
	| MASK_ mask
	| MOSAIC_ FAST_ yesno {FR->mosaicFastCmd($3);}
	| NAN_ COLOR_ STRING {FR->nanColorCmd($3);}
	| ORIENT_ orient
	| PAN_ pan
	| PANNER_ panner
	| POSTSCRIPT_ postscript
	| RESET_ {FR->resetCmd();}
	| REGION_ region
	| RGB_ rgb
	| ROTATE_ rotate
	| SAVE_ save
	| SHOW_ {FR->showCmd();}
	| SMOOTH_ smooth
	| UNLOAD_ {FR->unloadFitsCmd();}
	| UPDATE_ update
	| VERSION_ {FR->msg("Frame 1.0");}
	| WARP_ warp
	| WCS_ wcs
	| ZOOM_ zoom
	;

numeric	: REAL {$$=$1;}
	| INT {$$=$1;}
	;

debug	: yesno {yydebug=$1;}
	| AST_ yesno {DebugAST=$2;}
	| MOSAIC_ yesno {DebugMosaic=$2;}
	| PARSER_ yesno {yydebug=$2;}
	| PERF_ yesno {DebugPerf=$2;}
	| WCS_ yesno {DebugWCS=$2;}
	| BIN_ yesno {DebugBin=$2;}
	| GZ_ yesno {DebugGZ=$2;}
	| RGB_ yesno {DebugRGB=$2;}
	;

yesno	: INT {$$=($1 ? 1 : 0);}

	| YES_ {$$=1;}
	| 'Y' {$$=1;}
	| ON_ {$$=1;}
	| TRUE_ {$$=1;}

	| NO_ {$$=0;}
	| 'N' {$$=0;}
	| OFF_ {$$=0;}
	| FALSE_ {$$=0;}
	;

precision : /* empty */ {$$ = FrameBase::DEFAULT;}
	| DEFAULT_ {$$ = FrameBase::DEFAULT;}
	| FIXED_ {$$ = FrameBase::FIXED;}
	| SCIENTIFIC_ {$$ = FrameBase::SCIENTIFIC;}
	| INTEGER_ {$$ = FrameBase::INTEGER;}
	;

fileNameType : /* empty */ {$$ = FrameBase::ROOTBASE;}
	| ROOT_ BASE_ {$$ = FrameBase::ROOTBASE;}
	| FULL_ BASE_ {$$ = FrameBase::FULLBASE;}
	| ROOT_ {$$ = FrameBase::ROOT;}
	| FULL_ {$$ = FrameBase::FULL;}
	;

optangle : /* empty */ {$$ = 0;}
	| angle {$$ = $1;}
	;

angle	: numeric {$$ = degToRad($1);} /* assume degree */
	| DEGREE {$$ = degToRad($1);}
	| RADIAN {$$=$1;}
	;

sexagesimal: SEXSTR {$$ = parseDMS($1);}
	;

coord	: sexagesimal sexagesimal
	{
	  Vector r;
	  if (currentSky == GALACTIC || currentSky == ECLIPTIC) 
	    r = Vector($1,$2);
	  else
	    r = Vector($1*360./24.,$2);

	  $$[0] = r[0];
	  $$[1] = r[1];
	  $$[2] = r[2];
	}
	| HOUR MINUTE SECOND {frsign2 = frsign;} INT ARCMINUTE ARCSECOND
        {
	  Vector r = Vector(hmsToDegree(frsign2,$1,$2,$3), 
	    dmsToDegree(frsign,$5,$6,$7));

	  $$[0] = r[0];
	  $$[1] = r[1];
	  $$[2] = r[2];
	}
	| INT ARCMINUTE ARCSECOND {frsign2 = frsign;} INT ARCMINUTE ARCSECOND
        {
	  Vector r = Vector(dmsToDegree(frsign2,$1,$2,$3),
	    dmsToDegree(frsign,$5,$6,$7));

	  $$[0] = r[0];
	  $$[1] = r[1];
	  $$[2] = r[2];
	}
	| numeric numeric 
	{
	  $$[0] = $1;
	  $$[1] = $2;
	  $$[2] = 1;
	}
	;

coordSystem :IMAGE_ {$$ = currentCoord = IMAGE;}
	| PHYSICAL_ {$$ = currentCoord = PHYSICAL;}
	| DETECTOR_ {$$ = currentCoord = DETECTOR;}
	| AMPLIFIER_ {$$ = currentCoord = AMPLIFIER;}
	| wcsSystem {$$ = (CoordSystem)$1;}
	;

wcsSystem : WCS_ {$$ = currentCoord = WCS;}
	| WCSA_ {$$ = currentCoord = WCSA;}
	| WCSB_ {$$ = currentCoord = WCSB;}
	| WCSC_ {$$ = currentCoord = WCSC;}
	| WCSD_ {$$ = currentCoord = WCSD;}
	| WCSE_ {$$ = currentCoord = WCSE;}
	| WCSF_ {$$ = currentCoord = WCSF;}
	| WCSG_ {$$ = currentCoord = WCSG;}
	| WCSH_ {$$ = currentCoord = WCSH;}
	| WCSI_ {$$ = currentCoord = WCSI;}
	| WCSJ_ {$$ = currentCoord = WCSJ;}
	| WCSK_ {$$ = currentCoord = WCSK;}
	| WCSL_ {$$ = currentCoord = WCSL;}
	| WCSM_ {$$ = currentCoord = WCSM;}
	| WCSN_ {$$ = currentCoord = WCSN;}
	| WCSO_ {$$ = currentCoord = WCSO;}
	| WCSP_ {$$ = currentCoord = WCSP;}
	| WCSQ_ {$$ = currentCoord = WCSQ;}
	| WCSR_ {$$ = currentCoord = WCSR;}
	| WCSS_ {$$ = currentCoord = WCSS;}
	| WCST_ {$$ = currentCoord = WCST;}
	| WCSU_ {$$ = currentCoord = WCSU;}
	| WCSV_ {$$ = currentCoord = WCSV;}
	| WCSW_ {$$ = currentCoord = WCSW;}
	| WCSX_ {$$ = currentCoord = WCSX;}
	| WCSY_ {$$ = currentCoord = WCSY;}
	| WCSZ_ {$$ = currentCoord = WCSZ;}
	;

internalSystem : CANVAS_ {$$ = CANVAS;}
	| PANNER_ {$$ = PANNER;}
	;

skyFrame : /* empty */ {$$ = currentSky = FK5;}
	| FK4_ {$$ = currentSky = FK4;}
	| B1950_ {$$ = currentSky = FK4;}
	| FK5_ {$$ = currentSky = FK5;}
	| J2000_ {$$ = currentSky = FK5;}
	| ICRS_ {$$ = currentSky = ICRS;}
	| GALACTIC_ {$$ = currentSky = GALACTIC;}
	| ECLIPTIC_ {$$ = currentSky = ECLIPTIC;}
	;

skyFormat : /* empty */ {$$=DEGREES;}
	| DEGREES_ {$$=DEGREES;}
	| SEXAGESIMAL_ {$$=SEXAGESIMAL;}
	| HMS_ {$$=HMS;}
	| ARCMIN_ {$$=ARCMIN;}
	| ARCSEC_ {$$=ARCSEC;}
	;

scaleType: LINEAR_ {$$ = FrScale::LINEARSCALE;}
	| LOG_ {$$ = FrScale::LOGSCALE;}
	| POW_ {$$ = FrScale::POWSCALE;}
	| SQRT_ {$$ = FrScale::SQRTSCALE;}
	| SQUARED_ {$$ = FrScale::SQUAREDSCALE;}
	| HISTEQU_ {$$ = FrScale::HISTEQUSCALE;}
	;	

shmType	: /* empty */ {$$ = FrameBase::SHMID;}
	| SHMID_ {$$ = FrameBase::SHMID;}
	| KEY_ {$$ = FrameBase::KEY;}
	;

incrLoad: /* empty */ {$$ = FrameBase::LOADALL;}
	| ALL_ {$$ = FrameBase::LOADALL;}
	| INCR_ {$$ = FrameBase::INCR;}
	;

layerType : /* empty */ {$$ = FrameBase::IMG;}
	| IMAGE_ {$$ = FrameBase::IMG;}
	| MASK_ {$$ = FrameBase::MASK;}
	;

compressType: /* empty */ {$$ = FrameBase::NOCOMPRESS;}
	| NONE_ {$$ = FrameBase::NOCOMPRESS;}
	| GZ_ {$$ = FrameBase::GZ;}
	;

bin	: ABOUT_ numeric numeric {FR->binAboutCmd(Vector($2,$3));}
	| COLS_ STRING STRING STRING {FR->binColsCmd($2,$3,$4);}
	| DEPTH_ INT {FR->binDepthCmd($2);}
	| FACTOR_ binFactor
	| FUNCTION_ binFunction
	| BUFFER_ SIZE_ INT {FR->binBufferSizeCmd($3);}
	| TO_ binTo
	| FILTER_ STRING {FR->binFilterCmd($2);}
	;

binFactor : numeric {FR->binFactorCmd(Vector($1,$1));}
	| numeric numeric {FR->binFactorCmd(Vector($1,$2));}
	| numeric ABOUT_ numeric numeric
	  {FR->binFactorAboutCmd(Vector($1,$1), Vector($3,$4));}
	| numeric numeric ABOUT_ numeric numeric
	  {FR->binFactorAboutCmd(Vector($1,$2), Vector($4,$5));}
	| TO_ numeric {FR->binFactorToCmd(Vector($2,$2));}
	| TO_ numeric numeric {FR->binFactorToCmd(Vector($2,$3));}
	| TO_ numeric ABOUT_ numeric numeric
	  {FR->binFactorToAboutCmd(Vector($2,$2), Vector($4,$5));}
	| TO_ numeric numeric ABOUT_ numeric numeric
	  {FR->binFactorToAboutCmd(Vector($2,$3), Vector($5,$6));}
	;

binFunction : AVERAGE_ {FR->binFunctionCmd(FitsHist::AVERAGE);}
	| SUM_ {FR->binFunctionCmd(FitsHist::SUM);}
	;

binTo	: FIT_ {FR->binToFitCmd();}
	| numeric numeric ABOUT_ CENTER_ STRING STRING STRING 
	  {FR->binCmd(Vector($1,$2), $5, $6, $7);}
	| numeric numeric INT numeric numeric ABOUT_ CENTER_ 
	    STRING STRING STRING STRING
	  {FR->binCmd(Vector($1,$2), $3, Vector($4,$5), $8, $9, $10, $11);}
	| numeric numeric ABOUT_ numeric numeric STRING STRING STRING 
	  {FR->binCmd(Vector($1,$2), Vector($4,$5), $6, $7, $8);}
	| numeric numeric INT numeric numeric ABOUT_ numeric numeric 
	    STRING STRING STRING STRING
	  {FR->binCmd(Vector($1,$2), $3, Vector($4,$5), Vector($7,$8), 
	    $9, $10, $11, $12);}
	;

clip	: SCOPE_ clipScope
	| MODE_ clipMode
	| MINMAX_ clipMinMax
	| USER_ clipUser
	| ZSCALE_ PARAM_ numeric INT INT {FR->clipZScaleParamCmd($3,$4,$5);}
	| PRESERVE_ yesno {FR->clipPreserveCmd($2);}
	;

clipUser: /* empty */ numeric numeric {FR->clipUserCmd($1,$2);}
	| LOW_ numeric {FR->clipUserLowCmd($2);} 
	| HIGH_ numeric {FR->clipUserHighCmd($2);} 
	;

clipScope: GLOBAL_ {FR->clipScopeCmd(FrScale::GLOBAL);}
	| LOCAL_ {FR->clipScopeCmd(FrScale::LOCAL);}
	;

clipMode: numeric {FR->clipModeCmd($1);}
	| MINMAX_ {FR->clipModeCmd(FrScale::MINMAX);}
	| ZSCALE_ {FR->clipModeCmd(FrScale::ZSCALE);}
	| ZMAX_ {FR->clipModeCmd(FrScale::ZMAX);}
	| USER_ {FR->clipModeCmd(FrScale::USERCLIP);}
	;

clipMinMax : MODE_ clipMinMaxMode
	| SAMPLE_ INT {FR->clipMinMaxParamCmd($2);}
	;

clipMinMaxMode : AUTO_ {FR->clipMinMaxModeCmd(FrScale::AUTOSCAN);}
	| SCAN_ {FR->clipMinMaxModeCmd(FrScale::SCAN);}
	| SAMPLE_ {FR->clipMinMaxModeCmd(FrScale::SAMPLE);}
	| DATAMIN_ {FR->clipMinMaxModeCmd(FrScale::DATAMIN);}
	| IRAFMIN_ {FR->clipMinMaxModeCmd(FrScale::IRAFMIN);}
	;

colormap : INT numeric numeric INT POINTER POINTER INT 
	  {FR->colormapCmd($1, $2, $3, $4, (unsigned short*)$5, 
	    (unsigned char*)$6, $7);}
	| RGB_ numeric numeric numeric numeric numeric numeric INT POINTER INT
	  {FR->colormapCmd($2,$3,$4,$5,$6,$7,$8,(unsigned char*)$9,$10);}
	| AREA_ INT colormapMode 
	  {FR->colormapAreaCmd($2,(FrameBase::ColormapMode)$3);}
	| BEGIN_ colormapBegin
	| MOTION_ colormapMotion
	| END_ colormapEnd
	;

colormapBegin : /* empty */ {FR->colormapBeginCmd();}
	| numeric numeric {FR->colormapBeginCmd(Vector($1,$2));}
	;

colormapMotion: INT numeric numeric INT POINTER POINTER INT
	  {FR->colormapMotionCmd($1, $2, $3, $4, (unsigned short*)$5,
	    (unsigned char*)$6, $7);}
	| RGB_ numeric numeric numeric numeric numeric numeric INT POINTER INT
	  {FR->colormapMotionCmd($2,$3,$4,$5,$6,$7,$8,
	    (unsigned char*)$9,$10);}

colormapEnd: INT numeric numeric INT POINTER POINTER INT
	  {FR->colormapEndCmd($1, $2, $3, $4, (unsigned short*)$5,
	    (unsigned char*)$6, $7);}
	| RGB_ numeric numeric numeric numeric numeric numeric INT POINTER INT
	  {FR->colormapEndCmd($2,$3,$4,$5,$6,$7,$8,(unsigned char*)$9,$10);}

colormapMode: CENTER_ {$$ = FrameBase::CENTER;}
	| CLICK_ {$$ = FrameBase::CLICK;}
	;

contour	: COLOR_ STRING {FR->contourSetColorCmd($2);}
	| COPY_ coordSystem skyFrame 
	  {FR->contourCopyCmd((CoordSystem)$2, (SkyFrame)$3);}
	| CREATE_ contourCreate
	| DELETE_ {FR->contourDeleteCmd();}
	| DELETE_ ALL_ {FR->contourDeleteAllCmd();}
	| LOAD_ STRING INT STRING coordSystem skyFrame
	  {FR->contourLoadCmd($2, $3, $4, (CoordSystem)$5, (SkyFrame)$6);}
	| PASTE_ STRING INT POINTER coordSystem skyFrame
	  {FR->contourPasteCmd($2, $3, $4, (CoordSystem)$5, (SkyFrame)$6);}
	| SAVE_ STRING coordSystem skyFrame
	  {FR->contourSaveCmd($2, (CoordSystem)$3, (SkyFrame)$4);}
	| WIDTH_ INT {FR->contourSetLineWidthCmd($2);}
	;

contourCreate : STRING INT INT numeric contourmethod 
	  {FR->contourCreateCmd($1,$2,$3,$4,$5);}
	| STRING INT numeric contourmethod STRING
	  {FR->contourCreateCmd($1,$2,$3,$4,$5);}
	| POLYGON_ markerProperties
	  {FR->createContourPolygonCmd(currentColor,currentWidth, 
	    currentFont, currentText, currentProps, NULL, taglist,cblist);}
	;

contourmethod : SMOOTH_ {$$ = FVContour::SMOOTH;}
	| BLOCK_ {$$ = FVContour::BLOCK;}
	;

crosshair: BEGIN_ MOTION_ internalSystem numeric numeric
	  {FR->crosshairBeginCmd(Vector($4,$5), (InternalSystem)$3);}
	| MOTION_ internalSystem numeric numeric
	  {FR->crosshairMotionCmd(Vector($3,$4), (InternalSystem)$2);}
	| internalSystem numeric numeric
	  {FR->crosshairCmd(Vector($2,$3), (InternalSystem)$1);}
	| BEGIN_ MOTION_ coordSystem coord
	  {FR->crosshairBeginCmd(Vector($4), (CoordSystem)$3);}
	| MOTION_ coordSystem coord
	  {FR->crosshairMotionCmd(Vector($3), (CoordSystem)$2);}
	| coordSystem skyFrame coord
	  {FR->crosshairCmd(Vector($3), (CoordSystem)$1, (SkyFrame)$2);}
	| yesno {FR->crosshairCmd($1);}
	| WARP_ numeric numeric {FR->crosshairWarpCmd(Vector($2,$3));}
	;

export	: FITS_ STRING 
	| JPEG_ STRING numeric {FR->exportJPEGCmd($2,$3);}
	| TIFF_ STRING
	| PNG_ STRING
	| PPM_ STRING
	;

fitsy	: HAS_ EXT_ STRING {FR->fitsyHasExtCmd($3);}
	;

get	: BG_ COLOR_ {FR->getBgColorCmd();}
	| BIN_ getBin
	| CLIP_ getClip
	| COLORMAP_ {FR->getColormapCmd();}
	| COLORSCALE_ getColorScale
	| CONTOUR_ getContour
	| COORDINATES_ getCoord
	| CROSSHAIR_ getCrosshair
	| CURSOR_ getCursor
	| DATA_ getData
	| DATASEC_ {FR->getDATASECCmd();}
	| FITS_ getFits
	| GRID_ getGrid
	| HISTOGRAM_ STRING STRING {FR->getHistogramCmd($2,$3);}
	| HORIZONTAL_ CUT_ STRING STRING numeric numeric internalSystem
	  {FR->getHorzCutCmd($3,$4,Vector($5,$6),(InternalSystem)$7);}
	| IIS_ getiis
	| INFO_ getInfo
	| MINMAX_ {FR->getMinMaxCmd();}
	| MARKER_ markerLayer markerGet
	| MASK_ {FR->getMaskCmd();}
	| NAN_ COLOR_ {FR->getNANColorCmd();}
	| ORIENT_ {FR->getOrientCmd();}
	| PAN_ getCursor
	| PIXEL_ TABLE_ internalSystem numeric numeric INT STRING
	  {FR->getPixelTableCmd(Vector($4,$5), (InternalSystem)$3, $6, $7);}
	| RGB_ getRGB
	| ROTATE_ getRotate
	| SMOOTH_ getSmooth
	| TYPE_ {FR->getTypeCmd();}
	| VALUE_ internalSystem numeric numeric
	  {FR->getValueCmd(Vector($3,$4),(InternalSystem)$2);}
	| VERTICAL_ CUT_ STRING STRING numeric numeric internalSystem
	  {FR->getVertCutCmd($3,$4,Vector($5,$6),(InternalSystem)$7);}
	| WCS_ getWCS
	| ZOOM_ getZoom
	;

getBin	: DEPTH_ {FR->getBinDepthCmd();}
	| FACTOR_ {FR->getBinFactorCmd();}
	| FUNCTION_ {FR->getBinFunctionCmd();}
	| BUFFER_ SIZE_ {FR->getBinBufferSizeCmd();}
	| CURSOR_ {FR->getBinCursorCmd();}
	| FILTER_ {FR->getBinFilterCmd();}
	| COLS_ {FR->getBinColsCmd();}
	| COLS_ MINMAX_ STRING {FR->getBinColsMinMaxCmd($3);}
	| COLS_ DIM_ STRING {FR->getBinColsDimCmd($3);}
	| LIST_ {FR->getBinListCmd();}
	;

getClip	: getClipMode
	| SCOPE_ {FR->getClipScopeCmd();}
	| MODE_ {FR->getClipModeCmd();}
	| MINMAX_ getClipMinMax
	| USER_ LEVEL_ {FR->getClipUserCmd();}
	| ZSCALE_ PARAM_ {FR->getClipZScaleParamCmd();}
	| PRESERVE_ {FR->getClipPreserveCmd();}
	;

getClipMode: /* empty */ {FR->getClipCmd();}
	| numeric {FR->getClipCmd($1);}
	| MINMAX_ {FR->getClipCmd(FrScale::MINMAX);}
	| ZSCALE_ {FR->getClipCmd(FrScale::ZSCALE);}
	| ZMAX_ {FR->getClipCmd(FrScale::ZMAX);}
	| USER_ {FR->getClipCmd(FrScale::USERCLIP);}
	;

getClipMinMax : MODE_ {FR->getClipMinMaxModeCmd();}
	| SAMPLE_ {FR->getClipMinMaxSampleCmd();}
	;

getColorScale : /* empty */ {FR->getColorScaleCmd();}
	| LEVEL_ getColorScaleLevel
	;

getColorScaleLevel: INT {FR->getColorScaleLevelCmd($1);}
	| INT internalSystem numeric numeric
	  {FR->getColorScaleLevelCmd($1,Vector($3,$4),(InternalSystem)$2);}
	| INT numeric numeric scaleType 
	  {FR->getColorScaleLevelCmd($1,$2,$3,(FrScale::ColorScaleType)$4);}
	;

getContour: coordSystem skyFrame 
	  {FR->getContourCmd((CoordSystem)$1,(SkyFrame)$2);}
	| COLOR_ {FR->getContourColorNameCmd();}
	| LEVEL_ {FR->getContourLevelCmd();}
	| METHOD_ {FR->getContourMethodCmd();}
	| WIDTH_ {FR->getContourLineWidthCmd();}
	;

getCoord : internalSystem numeric numeric coordSystem skyFrame skyFormat
	  {FR->getCoordCmd(Vector($2,$3), (InternalSystem)$1, (CoordSystem)$4,
	    (SkyFrame)$5, (SkyFormat)$6);}
	;

getCrosshair: internalSystem {FR->getCrosshairCmd((InternalSystem)$1);}
	| coordSystem skyFrame skyFormat precision
	  {FR->getCrosshairCmd((CoordSystem)$1, (SkyFrame)$2, (SkyFormat)$3,
	    (FrameBase::Precision)$4)}
	| STATUS_ {FR->getCrosshairStatusCmd();}
	;

getCursor : internalSystem {FR->getCursorCmd((InternalSystem)$1);}
	| coordSystem skyFrame skyFormat precision
	  {FR->getCursorCmd((CoordSystem)$1, (SkyFrame)$2, (SkyFormat)$3,
	    (FrameBase::Precision)$4);}
	| PRESERVE_ {FR->getPanPreserveCmd();}
	;

getData	: coordSystem skyFrame coord numeric numeric STRING
	  {FR->getDataValuesCmd(1, Vector($3), 
	    (CoordSystem)$1, (SkyFrame)$2, Vector($4,$5), $6);}
	| INT coordSystem skyFrame coord numeric numeric STRING
	  {FR->getDataValuesCmd($1, Vector($4), 
	    (CoordSystem)$2, (SkyFrame)$3, Vector($5,$6), $7);}
	| internalSystem numeric numeric INT INT
	  {FR->getDataValuesCmd(Vector($2,$3),(InternalSystem)$1,
	    Vector($4,$5));}
	;

getInfo	: STRING {FR->getInfoCmd($1);}
	| CLIP_ {FR->getInfoClipCmd();}
 	| internalSystem numeric numeric skyFrame skyFormat STRING
	  {FR->getInfoCmd(Vector($2,$3), (InternalSystem)$1, 
	    (SkyFrame)$4, (SkyFormat)$5, $6);}
	;

getiis	: POINTER INT INT INT INT {FR->iisGetCmd((char*)$1,$2,$3,$4,$5);}
	| CURSOR_ {FR->iisGetCursorCmd();}
	| WIDTH_ {FR->getWidthCmd();}
	| HEIGHT_ {FR->getHeightCmd();}
	| FILE_ NAME_ getIISFileName
	;

getIISFileName : /* empty */ {FR->iisGetFileNameCmd();}
	| INT {FR->iisGetFileNameCmd($1);}
	| numeric numeric {FR->iisGetFileNameCmd(Vector($1,$2));}
	;

getFits	: COUNT_ {FR->getFitsCountCmd();}
	| MOSAIC_ COUNT_ {FR->getFitsMosaicCountCmd();}
	| DEPTH_ {FR->getFitsDepthCmd();}
	| BITPIX_ {FR->getBitpixCmd();}
	| EXT_ internalSystem numeric numeric
	  {FR->getFitsExtCmd(Vector($3,$4),(InternalSystem)$2);}
	| FILE_ NAME_ getFitsFileName
	| HEADER_ INT {FR->getFitsHeaderCmd($2);}
	| HEADER_ INT KEYWORD_ STRING {FR->getFitsHeaderKeywordCmd($2,$4);}
	| HEIGHT_ {FR->getHeightCmd();}
	| OBJECT_ NAME_ {FR->getFitsObjectNameCmd();}
	| MASTER_ {FR->getFitsMasterCmd();}
	| SIZE_ {FR->getFitsSizeCmd();}
	| SIZE_ coordSystem skyFrame skyFormat
	    {FR->getFitsSizeCmd((CoordSystem)$2,(SkyFrame)$3,(SkyFormat)$4);}
	| SLICE_ {FR->getFitsSliceCmd();}
	| WIDTH_ {FR->getWidthCmd();}
	;

getFitsFileName: fileNameType 
	  {FR->getFitsFileNameCmd((FrameBase::FileNameType)$1);}
	| fileNameType internalSystem numeric numeric
	  {FR->getFitsFileNameCmd(Vector($3,$4), (InternalSystem)$2,
	    (FrameBase::FileNameType)$1);}
	| fileNameType INT 
	  {FR->getFitsFileNameCmd($2, (FrameBase::FileNameType)$1);}
	;

getGrid : /* empty */ {FR->getGridCmd();}
	| OPTION_ {FR->getGridOptionCmd();}
	;

getRGB	: CHANNEL_ {FR->getRGBChannelCmd();}
	| SYSTEM_ {FR->getRGBSystemCmd();}
	| VIEW_ {FR->getRGBViewCmd();}
	;

getRotate : precision {FR->getRotateCmd((FrameBase::Precision)$1);}
	;

getSmooth : FUNCTION_  {FR->getSmoothFunctionCmd();}
	| RADIUS_ {FR->getSmoothRadiusCmd();}
	;

getWCS	: ALIGN_ {FR->getWCSAlignCmd();}
	| NAME_ wcsSystem {FR->getWCSNameCmd((CoordSystem)$2);}
	;

getZoom	: precision {FR->getZoomCmd((FrameBase::Precision)$1);}
	;

grid	: CREATE_ coordSystem skyFrame skyFormat gridType STRING
	   {FR->gridCmd((CoordSystem)$2, (SkyFrame)$3, (SkyFormat)$4,
	     (Grid::GridType)$5, $6);}
	| DELETE_ {FR->gridDeleteCmd();}
	;

gridType : ANALYSIS_ {$$=Grid::ANALYSIS;}
	| PUBLICATION_ {$$=Grid::PUBLICATION;}
	;

has	: AMPLIFIER_ {FR->hasAmplifierCmd();}
	| BIN_ {FR->hasBinCmd();}
	| CONTOUR_ {FR->hasContourCmd();}
	| DATAMIN_ {FR->hasDATAMINCmd();}
	| DATASEC_ {FR->hasDATASECCmd();}
	| DETECTOR_ {FR->hasDetectorCmd();}
	| FITS_ hasFits
	| GRID_ {FR->hasGridCmd();}
	| IIS_ {FR->hasIISCmd();}
	| IRAFMIN_ {FR->hasIRAFMINCmd();}
	| MARKER_ hasMarker
	| PHYSICAL_ {FR->hasPhysicalCmd();}
	| SMOOTH_ {FR->hasSmoothCmd();}
	| SYSTEM_ coordSystem {FR->hasSystemCmd((CoordSystem)$2);}
	| WCS_ hasWCS
	;

hasFits	: /* empty */ {FR->hasFitsCmd();}
	| CUBE_ {FR->hasFitsCubeCmd();}
	| IMAGE_ {FR->hasFitsImageCmd();}
	| MOSAIC_ {FR->hasFitsMosaicCmd();}
	| TABLE_ {FR->hasFitsTableCmd();}
	;

hasMarker : HIGHLITE_ {FR->hasMarkerHighlitedCmd();}
	| SELECT_ {FR->hasMarkerSelectedCmd();}
	| PASTE_ {FR->hasMarkerPasteCmd();}
	| UNDO_ {FR->hasMarkerUndoCmd();}
	;

hasWCS	: coordSystem {FR->hasWCSCmd((CoordSystem)$1);}
	| EQUATORIAL_ coordSystem {FR->hasWCSEquCmd((CoordSystem)$2);}
	| LINEAR_ coordSystem {FR->hasWCSLinearCmd((CoordSystem)$2);}
	;

iis	: NEW_ INT INT {FR->iisCmd($2,$3);}
	| ERASE_ {FR->iisEraseCmd();}
	| MESSAGE_ STRING {FR->iisMessageCmd($2);}
	| CURSOR_ iiscursor
	| SET_ FILE_ NAME_ iisSetFileName
	| SET_ POINTER INT INT INT INT
	  {FR->iisSetCmd((const char*)$2,$3,$4,$5,$6);}
	| UPDATE_ {FR->iisUpdateCmd();}
	| WCS_ numeric numeric numeric numeric 
	  numeric numeric numeric numeric INT
	    {FR->iisWCSCmd(Matrix($2,$3,$4,$5,$6,$7),Vector($8,$9),$10);}
	;

iisSetFileName : STRING {FR->iisSetFileNameCmd($1);}
	| STRING INT {FR->iisSetFileNameCmd($1,$2);}
	;

iiscursor: INT INT CANVAS_
	  {FR->iisSetCursorCmd(Vector($1,$2),CANVAS);}
	| INT INT coordSystem
	  {FR->iisSetCursorCmd(Vector($1,$2),(CoordSystem)$3);}
	| MODE_ yesno {FR->iisCursorModeCmd($2);}
	;

load	: ARRAY_ loadArr
	| FITS_ loadFits
	| INCR_ loadIncr
	;

loadArr : STRING ALLOC_ STRING layerType 
	  {FR->loadArrAllocCmd($3, $1, (FrameBase::LayerType)$4);}
	| STRING ALLOCGZ_ STRING layerType 
	  {FR->loadArrAllocGZCmd($3, $1, (FrameBase::LayerType)$4);}
	| STRING CHANNEL_ STRING layerType 
	  {FR->loadArrChannelCmd($3, $1, (FrameBase::LayerType)$4);}
	| STRING MMAP_ layerType 
	  {FR->loadArrMMapCmd($1, (FrameBase::LayerType)$3);}
	| STRING MMAPINCR_ layerType 
	  {FR->loadArrMMapIncrCmd($1, (FrameBase::LayerType)$3);}
	| STRING SHARED_ shmType INT layerType
	  {FR->loadArrShareCmd((FrameBase::ShmType)$3, $4, $1,
	    (FrameBase::LayerType)$5);}
	| STRING SOCKET_ INT layerType 
	  {FR->loadArrSocketCmd($3, $1, (FrameBase::LayerType)$4);}
	| STRING SOCKETGZ_ INT layerType 
	  {FR->loadArrSocketGZCmd($3, $1, (FrameBase::LayerType)$4);}
	| STRING VAR_ STRING layerType 
	  {FR->loadArrVarCmd($3, $1, (FrameBase::LayerType)$4);}
	| RGB_ CUBE_ loadArrRGBCube
	;

loadArrRGBCube : STRING ALLOC_ STRING {FR->loadArrRGBCubeAllocCmd($3, $1);}
	| STRING ALLOCGZ_ STRING {FR->loadArrRGBCubeAllocGZCmd($3, $1);}
	| STRING CHANNEL_ STRING {FR->loadArrRGBCubeChannelCmd($3, $1);}
	| STRING MMAP_ {FR->loadArrRGBCubeMMapCmd($1);}
	| STRING MMAPINCR_  {FR->loadArrRGBCubeMMapIncrCmd($1);}
	| STRING SHARED_ shmType INT
	  {FR->loadArrRGBCubeShareCmd((FrameBase::ShmType)$3, $4, $1);}
	| STRING SOCKET_ INT {FR->loadArrRGBCubeSocketCmd($3, $1);}
	| STRING SOCKETGZ_ INT {FR->loadArrRGBCubeSocketGZCmd($3, $1);}
	| STRING VAR_ STRING {FR->loadArrRGBCubeVarCmd($3, $1);}
	;

loadFits: STRING ALLOC_ STRING layerType 
	  {FR->loadFitsAllocCmd($3, $1, (FrameBase::LayerType)$4);}
	| STRING ALLOCGZ_ STRING layerType 
	  {FR->loadFitsAllocGZCmd($3, $1, (FrameBase::LayerType)$4);}
	| STRING CHANNEL_ STRING layerType 
	  {FR->loadFitsChannelCmd($3, $1, (FrameBase::LayerType)$4);}
	| STRING MMAP_ incrLoad layerType
	  {FR->loadFitsMMapCmd($1,(FrameBase::LoadMethod)$3,
	    (FrameBase::LayerType)$4);}
	| STRING STRING SMMAP_ incrLoad layerType
	  {FR->loadFitsSMMapCmd($1, $2,(FrameBase::LoadMethod)$4,	
	    (FrameBase::LayerType)$5);}
	| STRING MMAPINCR_ incrLoad layerType
	  {FR->loadFitsMMapIncrCmd($1,(FrameBase::LoadMethod)$3,
	    (FrameBase::LayerType)$4);}
	| STRING SHARED_ shmType INT incrLoad layerType
	  {FR->loadFitsShareCmd((FrameBase::ShmType)$3, $4, $1, 
	    (FrameBase::LoadMethod)$5,(FrameBase::LayerType)$6);}
	| STRING SSHARED_ shmType INT INT incrLoad layerType
	  {FR->loadFitsSShareCmd((FrameBase::ShmType)$3, $4, $5, $1, 
	    (FrameBase::LoadMethod)$6,(FrameBase::LayerType)$7);}
	| STRING SOCKET_ INT  layerType
	  {FR->loadFitsSocketCmd($3, $1, (FrameBase::LayerType)$4);}
	| STRING SOCKETGZ_ INT  layerType
	  {FR->loadFitsSocketGZCmd($3, $1, (FrameBase::LayerType)$4);}
	| STRING VAR_ STRING incrLoad layerType
	  {FR->loadFitsVarCmd($3, $1, (FrameBase::LoadMethod)$4,
	    (FrameBase::LayerType)$5);}
	| DATA_ CUBE_ loadFitsDataCube
	| MOSAIC_ loadFitsMosaic
	| RGB_ loadFitsRGB
	| SLAVE_ STRING POINTER {FR->loadFitsSlaveCmd($2,(FitsFile*)$3);}
	;

loadFitsDataCube: STRING ALLOC_ STRING layerType
	  {FR->loadDataCubeAllocCmd($3, $1, (FrameBase::LayerType)$4);}
	| STRING ALLOCGZ_ STRING layerType 
	  {FR->loadDataCubeAllocGZCmd($3, $1, (FrameBase::LayerType)$4);}
	| STRING CHANNEL_ STRING layerType
	  {FR->loadDataCubeChannelCmd($3, $1, (FrameBase::LayerType)$4);}
	| STRING MMAP_ incrLoad layerType
	  {FR->loadDataCubeMMapCmd($1, 
	    (FrameBase::LoadMethod)$3, (FrameBase::LayerType)$4);}
	| STRING MMAPINCR_ incrLoad layerType
	  {FR->loadDataCubeMMapIncrCmd($1, 
	    (FrameBase::LoadMethod)$3, (FrameBase::LayerType)$4);}
	| STRING SHARED_ shmType INT incrLoad layerType
	  {FR->loadDataCubeShareCmd((FrameBase::ShmType)$3, $4, $1, 
	    (FrameBase::LoadMethod)$5, (FrameBase::LayerType)$6);}
	| STRING SOCKET_ INT layerType
	  {FR->loadDataCubeSocketCmd($3, $1, (FrameBase::LayerType)$4);}
	| STRING SOCKETGZ_ INT layerType
	  {FR->loadDataCubeSocketGZCmd($3, $1, (FrameBase::LayerType)$4);}
	| STRING VAR_ STRING incrLoad layerType
	  {FR->loadDataCubeVarCmd($3, $1, 
	    (FrameBase::LoadMethod)$4, (FrameBase::LayerType)$5);}
	;

loadFitsMosaic : IMAGE_ IRAF_ loadFitsMosaicImageIRAF
	| IRAF_ loadFitsMosaicIRAF
	| IMAGE_ loadFitsMosaicImageWCS
	| loadFitsMosaicWCS
	| IMAGE_ WFPC2_ loadFitsMosaicImageWFPC2
	;

loadFitsMosaicImageIRAF : STRING ALLOC_ STRING layerType
	  {FR->loadMosaicImageIRAFAllocCmd($3, $1, (FrameBase::LayerType)$4);}
	| STRING ALLOCGZ_ STRING layerType
	  {FR->loadMosaicImageIRAFAllocGZCmd($3, $1,(FrameBase::LayerType)$4);}
	| STRING CHANNEL_ STRING layerType
	  {FR->loadMosaicImageIRAFChannelCmd($3, $1,(FrameBase::LayerType)$4);}
	| STRING MMAP_ incrLoad layerType
	  {FR->loadMosaicImageIRAFMMapCmd($1,(FrameBase::LoadMethod)$3,
	    (FrameBase::LayerType)$4);}
	| STRING MMAPINCR_ incrLoad layerType
	  {FR->loadMosaicImageIRAFMMapIncrCmd($1,(FrameBase::LoadMethod)$3,
	    (FrameBase::LayerType)$4);}
	| STRING SHARED_ shmType INT incrLoad layerType
	  {FR->loadMosaicImageIRAFShareCmd((FrameBase::ShmType)$3, $4,
	    $1, (FrameBase::LoadMethod)$5, (FrameBase::LayerType)$6);}
	| STRING SOCKET_ INT layerType
	  {FR->loadMosaicImageIRAFSocketCmd($3, $1, (FrameBase::LayerType)$4);}
	| STRING SOCKETGZ_ INT layerType
	  {FR->loadMosaicImageIRAFSocketGZCmd($3, $1, 
	    (FrameBase::LayerType)$4);}
	| STRING VAR_ STRING incrLoad layerType
	  {FR->loadMosaicImageIRAFVarCmd($3, $1, (FrameBase::LoadMethod)$4, 
	    (FrameBase::LayerType)$5);}
	;

loadFitsMosaicIRAF : STRING ALLOC_ STRING layerType
	  {FR->loadMosaicIRAFAllocCmd($3, $1, (FrameBase::LayerType)$4);}
	| STRING ALLOCGZ_ STRING layerType
	  {FR->loadMosaicIRAFAllocGZCmd($3, $1, (FrameBase::LayerType)$4);}
	| STRING CHANNEL_ STRING layerType
	  {FR->loadMosaicIRAFChannelCmd($3, $1, (FrameBase::LayerType)$4);}
	| STRING MMAP_ incrLoad layerType
	  {FR->loadMosaicIRAFMMapCmd($1,(FrameBase::LoadMethod)$3, 
	    (FrameBase::LayerType)$4);}
	| STRING STRING SMMAP_ incrLoad layerType
	  {FR->loadMosaicIRAFSMMapCmd($1, $2,(FrameBase::LoadMethod)$4, 
	    (FrameBase::LayerType)$5);}
	| STRING MMAPINCR_ incrLoad layerType
	  {FR->loadMosaicIRAFMMapIncrCmd($1,(FrameBase::LoadMethod)$3, 
	    (FrameBase::LayerType)$4);}
	| STRING SHARED_ shmType INT incrLoad layerType
	  {FR->loadMosaicIRAFShareCmd((FrameBase::ShmType)$3, $4,
	    $1, (FrameBase::LoadMethod)$5, (FrameBase::LayerType)$6);}
	| STRING SSHARED_ shmType INT INT incrLoad layerType
	  {FR->loadMosaicIRAFSShareCmd((FrameBase::ShmType)$3, $4, $5,
	    $1, (FrameBase::LoadMethod)$6, (FrameBase::LayerType)$7);}
	| STRING SOCKET_ INT layerType
	  {FR->loadMosaicIRAFSocketCmd($3, $1, (FrameBase::LayerType)$4);}
	| STRING SOCKETGZ_ INT layerType
	  {FR->loadMosaicIRAFSocketGZCmd($3, $1, (FrameBase::LayerType)$4);}
	| STRING VAR_ STRING incrLoad layerType
	  {FR->loadMosaicIRAFVarCmd($3, $1, (FrameBase::LoadMethod)$4, 
	    (FrameBase::LayerType)$5);}
	;

loadFitsMosaicImageWCS : wcsSystem STRING ALLOC_ STRING layerType
	  {FR->loadMosaicImageWCSAllocCmd((CoordSystem)$1, $4, $2, 
	    (FrameBase::LayerType)$5);}
	| wcsSystem STRING ALLOCGZ_ STRING layerType
	  {FR->loadMosaicImageWCSAllocGZCmd((CoordSystem)$1, $4, $2,
	    (FrameBase::LayerType)$5);}
	| wcsSystem STRING CHANNEL_ STRING layerType
	  {FR->loadMosaicImageWCSChannelCmd((CoordSystem)$1, $4, $2,
  	    (FrameBase::LayerType)$5);}
	| wcsSystem STRING MMAP_ incrLoad layerType
	  {FR->loadMosaicImageWCSMMapCmd((CoordSystem)$1, $2,
	    (FrameBase::LoadMethod)$4, (FrameBase::LayerType)$5);}
	| wcsSystem STRING MMAPINCR_ incrLoad layerType
	  {FR->loadMosaicImageWCSMMapIncrCmd((CoordSystem)$1, $2,
	    (FrameBase::LoadMethod)$4, (FrameBase::LayerType)$5);}
	| wcsSystem STRING SHARED_ shmType INT incrLoad layerType
	  {FR->loadMosaicImageWCSShareCmd((CoordSystem)$1, 
	    (FrameBase::ShmType)$4, $5, $2, (FrameBase::LoadMethod)$6,
	    (FrameBase::LayerType)$7);}
	| wcsSystem STRING SOCKET_ INT layerType
	  {FR->loadMosaicImageWCSSocketCmd((CoordSystem)$1, $4, $2,	
	    (FrameBase::LayerType)$5);}
	| wcsSystem STRING SOCKETGZ_ INT layerType
	  {FR->loadMosaicImageWCSSocketGZCmd((CoordSystem)$1, $4, $2,
	    (FrameBase::LayerType)$5);}
	| wcsSystem STRING VAR_ STRING incrLoad layerType
	  {FR->loadMosaicImageWCSVarCmd((CoordSystem)$1, $4, $2, 
	    (FrameBase::LoadMethod)$5, (FrameBase::LayerType)$6);}
	;

loadFitsMosaicWCS : wcsSystem STRING ALLOC_ STRING layerType
	  {FR->loadMosaicWCSAllocCmd((CoordSystem)$1, $4, $2, 
	    (FrameBase::LayerType)$5);}
	| wcsSystem STRING ALLOCGZ_ STRING layerType
	  {FR->loadMosaicWCSAllocGZCmd((CoordSystem)$1, $4, $2,
	    (FrameBase::LayerType)$5);}
	| wcsSystem STRING CHANNEL_ STRING  layerType
	  {FR->loadMosaicWCSChannelCmd((CoordSystem)$1, $4, $2,
	    (FrameBase::LayerType)$5);}
	| wcsSystem STRING MMAP_ incrLoad  layerType
	  {FR->loadMosaicWCSMMapCmd((CoordSystem)$1, $2,
	    (FrameBase::LoadMethod)$4, (FrameBase::LayerType)$5);}
	| wcsSystem STRING STRING SMMAP_ incrLoad  layerType
	  {FR->loadMosaicWCSSMMapCmd((CoordSystem)$1, $2, $3,
	    (FrameBase::LoadMethod)$5, (FrameBase::LayerType)$6);}
	| wcsSystem STRING MMAPINCR_ incrLoad  layerType
	  {FR->loadMosaicWCSMMapIncrCmd((CoordSystem)$1, $2,
	    (FrameBase::LoadMethod)$4, (FrameBase::LayerType)$5);}
	| wcsSystem STRING SHARED_ shmType INT incrLoad layerType
	  {FR->loadMosaicWCSShareCmd((CoordSystem)$1, 
	    (FrameBase::ShmType)$4, $5, $2, (FrameBase::LoadMethod)$6,
	    (FrameBase::LayerType)$7);}
	| wcsSystem STRING SSHARED_ shmType INT INT incrLoad layerType
	  {FR->loadMosaicWCSSShareCmd((CoordSystem)$1, 
	    (FrameBase::ShmType)$4, $5, $6, $2, (FrameBase::LoadMethod)$7,
	    (FrameBase::LayerType)$8);}
	| wcsSystem STRING SOCKET_ INT  layerType
	  {FR->loadMosaicWCSSocketCmd((CoordSystem)$1, $4, $2,
	    (FrameBase::LayerType)$5);}
	| wcsSystem STRING SOCKETGZ_ INT  layerType
	  {FR->loadMosaicWCSSocketGZCmd((CoordSystem)$1, $4, $2,
	    (FrameBase::LayerType)$5);}
	| wcsSystem STRING VAR_ STRING incrLoad layerType
	  {FR->loadMosaicWCSVarCmd((CoordSystem)$1, $4, $2, 
	    (FrameBase::LoadMethod)$5, (FrameBase::LayerType)$6);}
	;

loadFitsMosaicImageWFPC2 : STRING ALLOC_ STRING layerType
	  {FR->loadMosaicImageWFPC2AllocCmd($3, $1, (FrameBase::LayerType)$4);}
	| STRING ALLOCGZ_ STRING layerType
	  {FR->loadMosaicImageWFPC2AllocGZCmd($3, $1,
	    (FrameBase::LayerType)$4);}
	| STRING CHANNEL_ STRING layerType
	  {FR->loadMosaicImageWFPC2ChannelCmd($3, $1,
	    (FrameBase::LayerType)$4);}
	| STRING MMAP_ incrLoad  layerType
	  {FR->loadMosaicImageWFPC2MMapCmd($1,(FrameBase::LoadMethod)$3,
	    (FrameBase::LayerType)$4);}
	| STRING MMAPINCR_ incrLoad  layerType
	  {FR->loadMosaicImageWFPC2MMapIncrCmd($1,(FrameBase::LoadMethod)$3,
	    (FrameBase::LayerType)$4);}
	| STRING SHARED_ shmType INT incrLoad layerType
	  {FR->loadMosaicImageWFPC2ShareCmd((FrameBase::ShmType)$3, $4,
	    $1, (FrameBase::LoadMethod)$5, (FrameBase::LayerType)$6);}
	| STRING SOCKET_ INT layerType 
	  {FR->loadMosaicImageWFPC2SocketCmd($3, $1,
	    (FrameBase::LayerType)$4);}
	| STRING SOCKETGZ_ INT layerType 
	  {FR->loadMosaicImageWFPC2SocketGZCmd($3, $1,
	    (FrameBase::LayerType)$4);}
	| STRING VAR_ STRING incrLoad  layerType
	  {FR->loadMosaicImageWFPC2VarCmd($3, $1, (FrameBase::LoadMethod)$4,
	    (FrameBase::LayerType)$5);}
	;
loadFitsRGB : CUBE_ loadFitsRGBCube
	| IMAGE_ loadFitsRGBImage
	;

loadFitsRGBCube: STRING ALLOC_ STRING {FR->loadRGBCubeAllocCmd($3, $1);}
	| STRING ALLOCGZ_ STRING {FR->loadRGBCubeAllocGZCmd($3, $1);}
	| STRING CHANNEL_ STRING {FR->loadRGBCubeChannelCmd($3, $1);}
	| STRING MMAP_ incrLoad
	  {FR->loadRGBCubeMMapCmd($1, (FrameBase::LoadMethod)$3);}
	| STRING STRING SMMAP_ incrLoad
	  {FR->loadRGBCubeSMMapCmd($1, $2, (FrameBase::LoadMethod)$4);}
	| STRING MMAPINCR_ incrLoad
	  {FR->loadRGBCubeMMapIncrCmd($1, (FrameBase::LoadMethod)$3);}
	| STRING SHARED_ shmType INT incrLoad
	  {FR->loadRGBCubeShareCmd((FrameBase::ShmType)$3, $4, $1, 
	    (FrameBase::LoadMethod)$5);}
	| STRING SSHARED_ shmType INT INT incrLoad
	  {FR->loadRGBCubeSShareCmd((FrameBase::ShmType)$3, $4, $5, $1, 
	    (FrameBase::LoadMethod)$6);}
	| STRING SOCKET_ INT {FR->loadRGBCubeSocketCmd($3, $1);}
	| STRING SOCKETGZ_ INT {FR->loadRGBCubeSocketGZCmd($3, $1);}
	| STRING VAR_ STRING incrLoad
	  {FR->loadRGBCubeVarCmd($3, $1, (FrameBase::LoadMethod)$4);}

	| SLAVE_ STRING POINTER {FR->loadRGBCubeSlaveCmd($2,(FitsFile*)$3);}
	;

loadFitsRGBImage: STRING ALLOC_ STRING {FR->loadRGBImageAllocCmd($3, $1);}
	| STRING ALLOCGZ_ STRING {FR->loadRGBImageAllocGZCmd($3, $1);}
	| STRING CHANNEL_ STRING {FR->loadRGBImageChannelCmd($3, $1);}
	| STRING MMAP_ incrLoad
	  {FR->loadRGBImageMMapCmd($1, (FrameBase::LoadMethod)$3);}
	| STRING MMAPINCR_ incrLoad
	  {FR->loadRGBImageMMapIncrCmd($1, (FrameBase::LoadMethod)$3);}
	| STRING SHARED_ shmType INT incrLoad
	  {FR->loadRGBImageShareCmd((FrameBase::ShmType)$3, $4, $1, 
	    (FrameBase::LoadMethod)$5);}
	| STRING SOCKET_ INT {FR->loadRGBImageSocketCmd($3, $1);}
	| STRING SOCKETGZ_ INT {FR->loadRGBImageSocketGZCmd($3, $1);}
	| STRING VAR_ STRING incrLoad
	  {FR->loadRGBImageVarCmd($3, $1, (FrameBase::LoadMethod)$4);}
	;

loadIncr: DATA_ INT INT INT INT INT {FR->loadIncrDataCmd($2,$3,$4,$5,$6);}
	| MINMAX_ INT INT INT INT INT {FR->loadIncrMinMaxCmd($2,$3,$4,$5,$6);}
	| END_ {FR->loadIncrEndCmd();}
	;

magnifier: yesno {FR->magnifierCmd($1);}
	| GRAPHICS_ yesno {FR->magnifierGraphicsCmd($2);}
	| CURSOR_ yesno {FR->magnifierCursorCmd($2);}
	| STRING INT INT {FR->magnifierCmd($1, $2, $3);}
	| UPDATE_ {FR->updateMagnifierCmd();}
	| UPDATE_ numeric numeric {FR->updateMagnifierCmd(Vector($2, $3));}
	| ZOOM_ numeric {FR->magnifierZoomCmd($2);}
	;

marker	: COLOR_ STRING {FR->markerColorCmd($2);}
	| COPY_ {FR->markerCopyCmd();}
	| COMMAND_ markerFormat STRING
	  {FR->markerCommandCmd((FrameBase::MarkerFormat)$2,$3);}
	| COMMAND_ markerFormat VAR_ STRING
	  {FR->markerCommandVarCmd((FrameBase::MarkerFormat)$2,$4);}
	| COMPOSITE_ DELETE_ {FR->markerCompositeDeleteCmd();}
	| CREATE_ markerCreate
	| CUT_ {FR->markerCutCmd();}
	| DELETE_ {FR->markerDeleteCmd();}
	| DELETE_ ALL_ {FR->markerDeleteAllCmd();}
	| EDIT_ markerEdit
	| FONT_ STRING {FR->markerFontCmd($2);}

	| INT ANGLE_ angle {FR->markerAngleCmd($1,$3);}
	| INT ANGLE_ angle internalSystem {FR->markerAngleCmd($1,$3);}
	| INT ANGLE_ angle coordSystem skyFrame
	  {FR->markerAngleCmd($1,$3,(CoordSystem)$4, (SkyFrame)$5);}

	| INT ANNULUS_ RADIUS_ numeric numeric INT internalSystem
	  {FR->markerAnnulusRadiusCmd($1, $4, $5, $6, (InternalSystem)$7);}
	| INT ANNULUS_ RADIUS_ numeric numeric INT coordSystem skyFormat
	  {FR->markerAnnulusRadiusCmd($1, $4, $5, $6,
	    (CoordSystem)$7, (SkyFormat)$8);}
	| INT ANNULUS_ RADIUS_ STRING coordSystem skyFormat
	  {FR->markerAnnulusRadiusCmd($1, $4,(CoordSystem)$5,(SkyFormat)$6);}

	| INT BOXANNULUS_ RADIUS_ numeric numeric numeric INT internalSystem
	  {FR->markerBoxAnnulusRadiusCmd($1, Vector($4, $5), 
	    Vector($6, $6*$5/$4), $7, (InternalSystem)$8);}
	| INT BOXANNULUS_ RADIUS_ numeric numeric numeric INT 
	    coordSystem skyFormat
	  {FR->markerBoxAnnulusRadiusCmd($1, Vector($4, $5), 
	    Vector($6, $6*$5/$4), $7, (CoordSystem)$8, (SkyFormat)$9);}
	| INT BOXANNULUS_ RADIUS_ STRING coordSystem skyFormat
	  {FR->markerBoxAnnulusRadiusCmd($1,$4,(CoordSystem)$5,(SkyFormat)$6);}

	| INT BOX_ RADIUS_ numeric numeric internalSystem
	  {FR->markerBoxRadiusCmd($1, Vector($4,$5), (InternalSystem)$6);}
	| INT BOX_ RADIUS_ numeric numeric coordSystem skyFormat
	  {FR->markerBoxRadiusCmd($1, Vector($4,$5), 
	    (CoordSystem)$6, (SkyFormat)$7);}

	| INT BPANDA_ EDIT_ angle angle INT numeric numeric numeric INT 
	  {FR->markerBpandaEditCmd($1, $4, $5, $6,
	    Vector($7,$8), Vector($9,$9*$8/$7), $10);}
	| INT BPANDA_ EDIT_ angle angle INT numeric numeric numeric INT 
	    internalSystem
	  {FR->markerBpandaEditCmd($1, $4, $5, $6,
	    Vector($7,$8), Vector($9,$9*$8/$7), $10);}
	| INT BPANDA_ EDIT_ angle angle INT numeric numeric numeric INT 
	    coordSystem skyFrame
	  {FR->markerBpandaEditCmd($1, $4, $5, $6,
	    Vector($7,$8), Vector($9,$9*$8/$7), $10,
	    (CoordSystem)$11, (SkyFrame)$12);}
	| INT BPANDA_ EDIT_ STRING STRING coordSystem skyFrame 
	    coordSystem skyFormat
	  {FR->markerBpandaEditCmd($1, $4, $5, 
	    (CoordSystem)$6, (SkyFrame)$7, (CoordSystem)$8, (SkyFormat)$9);}

	| INT CALLBACK_ markerCallBack STRING STRING
	  {FR->markerCallBackCmd($1,(CallBack::Type)$3,$4,$5);}

	| INT CIRCLE_ RADIUS_ numeric internalSystem
	  {FR->markerCircleRadiusCmd($1, $4, (InternalSystem)$5);}
	| INT CIRCLE_ RADIUS_ numeric coordSystem skyFormat
	  {FR->markerCircleRadiusCmd($1, $4, (CoordSystem)$5, (SkyFormat)$6);}

	| INT CPANDA_ EDIT_ angle angle INT numeric numeric INT 
	  {FR->markerCpandaEditCmd($1, $4, $5, $6, $7, $8, $9);}
	| INT CPANDA_ EDIT_ angle angle INT numeric numeric INT internalSystem
	  {FR->markerCpandaEditCmd($1, $4, $5, $6, $7, $8, $9);}
	| INT CPANDA_ EDIT_ angle angle INT numeric numeric INT 
	    coordSystem skyFrame
	  {FR->markerCpandaEditCmd($1, $4, $5, $6, $7, $8, $9,
	    (CoordSystem)$10, (SkyFrame)$11);}
	| INT CPANDA_ EDIT_ STRING STRING coordSystem skyFrame 
	    coordSystem skyFormat
	  {FR->markerCpandaEditCmd($1, $4, $5, (CoordSystem)$6, (SkyFrame)$7,
	    (CoordSystem)$8, (SkyFormat)$9);}

	| INT COLOR_ STRING {FR->markerColorCmd($1,$3);}

	| INT COMPASS_ ARROW_ yesno yesno 
	  {FR->markerCompassArrowCmd($1,$4,$5);}
	| INT COMPASS_ LABEL_ STRING STRING
	  {FR->markerCompassLabelCmd($1,$4,$5);}
	| INT COMPASS_ RADIUS_ numeric internalSystem
	  {FR->markerCompassRadiusCmd($1,$4,(InternalSystem)$5);}
	| INT COMPASS_ RADIUS_ numeric coordSystem skyFormat
	  {FR->markerCompassRadiusCmd($1,$4,(CoordSystem)$5,(SkyFormat)$6);}
	| INT COMPASS_ SYSTEM_ coordSystem skyFrame
	  {FR->markerCompassSystemCmd($1, (CoordSystem)$4, (SkyFrame)$5);}

	| INT CREATE_ ANNULUS_ RADIUS_ numeric numeric
	  {FR->markerAnnulusCreateRadiusCmd($1,Vector($5,$6));}
	| INT CREATE_ BOXANNULUS_ RADIUS_ numeric numeric
	  {FR->markerBoxAnnulusCreateRadiusCmd($1,Vector($5,$6));}
	| INT CREATE_ BPANDA_ ANGLE_ numeric numeric
	  {FR->markerBpandaCreateAnglesCmd($1,Vector($5,$6));}
	| INT CREATE_ BPANDA_ RADIUS_ numeric numeric
	  {FR->markerBpandaCreateRadiusCmd($1,Vector($5,$6));}
	| INT CREATE_ ELLIPSEANNULUS_ RADIUS_ numeric numeric
	  {FR->markerEllipseAnnulusCreateRadiusCmd($1,Vector($5,$6));}
	| INT CREATE_ EPANDA_ ANGLE_ numeric numeric
	  {FR->markerEpandaCreateAnglesCmd($1,Vector($5,$6));}
	| INT CREATE_ EPANDA_ RADIUS_ numeric numeric
	  {FR->markerEpandaCreateRadiusCmd($1,Vector($5,$6));}
	| INT CREATE_ CPANDA_ ANGLE_ numeric numeric
	  {FR->markerCpandaCreateAnglesCmd($1,Vector($5,$6));}
	| INT CREATE_ CPANDA_ RADIUS_ numeric numeric
	  {FR->markerCpandaCreateRadiusCmd($1,Vector($5,$6));}
	| INT CREATE_ POLYGON_ VERTEX_ INT numeric numeric
	  {FR->markerPolygonCreateVertexCmd($1,$5,Vector($6,$7));}

	| INT DELETE_ {FR->markerDeleteCmd($1);}
	| INT DELETE_ ANNULUS_ INT 
	  {FR->markerAnnulusDeleteRadiusCmd($1,$4);}
	| INT DELETE_ BOXANNULUS_ INT 
	  {FR->markerBoxAnnulusDeleteRadiusCmd($1,$4);}
	| INT DELETE_ BPANDA_ INT {FR->markerBpandaDeleteCmd($1,$4);}
	| INT DELETE_ ELLIPSEANNULUS_ INT 
	  {FR->markerEllipseAnnulusDeleteRadiusCmd($1,$4);}
	| INT DELETE_ CALLBACK_ markerCallBack STRING
	  {FR->markerDeleteCallBackCmd($1,(CallBack::Type)$4,$5);}
	| INT DELETE_ EPANDA_ INT {FR->markerEpandaDeleteCmd($1,$4);}
	| INT DELETE_ CPANDA_ INT {FR->markerCpandaDeleteCmd($1,$4);}
	| INT DELETE_ POLYGON_ VERTEX_ INT 
	  {FR->markerPolygonDeleteVertexCmd($1,$5);}
	| INT DELETE_ TAG_ {FR->markerDeleteTagCmd($1);}
	| INT DELETE_ TAG_ STRING {FR->markerDeleteTagCmd($1,$4);}
	| INT DELETE_ TAG_ INT {FR->markerDeleteTagCmd($1,$4);}

	| INT EDIT_ BEGIN_ INT {FR->markerEditBeginCmd($1,$4);}

	| INT ELLIPSE_ RADIUS_ numeric numeric internalSystem
	  {FR->markerEllipseRadiusCmd($1, Vector($4, $5), (InternalSystem)$6);}
	| INT ELLIPSE_ RADIUS_ numeric numeric coordSystem skyFormat
	  {FR->markerEllipseRadiusCmd($1, Vector($4, $5),
	    (CoordSystem)$6, (SkyFormat)$7);}

	| INT ELLIPSEANNULUS_ RADIUS_ numeric numeric numeric INT
	    internalSystem
	  {FR->markerEllipseAnnulusRadiusCmd($1, Vector($4,$5), 
	    Vector($6,$6*$5/$4), $7, (InternalSystem)$8);}
	| INT ELLIPSEANNULUS_ RADIUS_ numeric numeric numeric INT
	    coordSystem skyFormat
	  {FR->markerEllipseAnnulusRadiusCmd($1, Vector($4,$5), 
	    Vector($6,$6*$5/$4), $7, (CoordSystem)$8, (SkyFormat)$9);}
	| INT ELLIPSEANNULUS_ RADIUS_ STRING coordSystem skyFormat
	  {FR->markerEllipseAnnulusRadiusCmd($1, $4,
	    (CoordSystem)$5, (SkyFormat)$6);}

	| INT EPANDA_ EDIT_ angle angle INT numeric numeric numeric INT 
	  {FR->markerEpandaEditCmd($1, $4, $5, $6,
	    Vector($7,$8), Vector($9,$9*$8/$7), $10);}
	| INT EPANDA_ EDIT_ angle angle INT numeric numeric numeric INT 
	    internalSystem
	  {FR->markerEpandaEditCmd($1, $4, $5, $6,
	    Vector($7,$8), Vector($9,$9*$8/$7), $10);}
	| INT EPANDA_ EDIT_ angle angle INT numeric numeric numeric INT 
	    coordSystem skyFrame
	  {FR->markerEpandaEditCmd($1, $4, $5, $6,
	    Vector($7,$8), Vector($9,$9*$8/$7), $10,
	    (CoordSystem)$11, (SkyFrame)$12);}
	| INT EPANDA_ EDIT_ STRING STRING coordSystem skyFrame 
	    coordSystem skyFormat
	  {FR->markerEpandaEditCmd($1, $4, $5, 
	    (CoordSystem)$6, (SkyFrame)$7,
	    (CoordSystem)$8, (SkyFormat)$9);}

	| INT FONT_ STRING {FR->markerFontCmd($1,$3);}
	| INT HIGHLITE_ {FR->markerHighliteCmd($1);}
	| INT HIGHLITE_ ONLY_{FR->markerHighliteOnlyCmd($1);}

	| INT LINE_ ARROW_ yesno yesno {FR->markerLineArrowCmd($1,$4,$5);}
	| INT LINE_ POINT_ internalSystem coord coord
	  {FR->markerLineCmd($1, Vector($5), Vector($6), (InternalSystem)$4);}
	| INT LINE_ POINT_ coordSystem skyFrame coord coord
	  {FR->markerLineCmd($1, Vector($6), Vector($7), 
	    (CoordSystem)$4, (SkyFrame)$5);}

	| INT MOVE_ numeric numeric 
	  {FR->markerMoveCmd($1, Vector($3,$4));}
	| INT MOVE_ FRONT_ {FR->markerFrontCmd($1);}
	| INT MOVE_ BACK_ {FR->markerBackCmd($1);}
	| INT MOVE_ TO_ internalSystem numeric numeric
	  {FR->markerMoveToCmd($1, Vector($5,$6), (InternalSystem)$4);}
	| INT MOVE_ TO_ coordSystem skyFrame coord 
	  {FR->markerMoveToCmd($1,Vector($6),(CoordSystem)$4,(SkyFrame)$5);}

	| INT POLYGON_ RESET_ numeric numeric internalSystem
	  {FR->markerPolygonResetCmd($1, Vector($4,$5),(InternalSystem)$6);}
	| INT POLYGON_ RESET_ numeric numeric coordSystem skyFormat
	  {FR->markerPolygonResetCmd($1, Vector($4,$5),
	    (CoordSystem)$6, (SkyFormat)$7);}

	| INT PROJECTION_ internalSystem coord coord numeric INT 
	  {FR->markerProjectionCmd($1, Vector($4), Vector($5),
	    (InternalSystem)$3, $6, $7);}
	| INT PROJECTION_ coordSystem skyFrame coord coord numeric INT 
	    coordSystem skyFormat
	  {FR->markerProjectionCmd($1, Vector($5), Vector($6),
	    (CoordSystem)$3, (SkyFrame)$4, $7, $8,
	    (CoordSystem)$9, (SkyFormat)$10);}
	| INT PROJECTION_ SYSTEM_ coordSystem skyFrame
	  {FR->markerProjectionSystemCmd($1,(CoordSystem)$4,(SkyFrame)$5);}

	| INT PROPERTY_ markerProperty yesno 
	  {FR->markerPropertyCmd($1,$3,$4);}
	| INT ROTATE_ BEGIN_ {FR->markerRotateBeginCmd($1);}

	| INT RULER_ POINT_ internalSystem coord coord
	  {FR->markerRulerPointCmd($1, Vector($5), Vector($6),
	    (InternalSystem)$4);}
	| INT RULER_ POINT_ coordSystem skyFrame coord coord
	  {FR->markerRulerPointCmd($1, Vector($6), Vector($7),
	    (CoordSystem)$4, (SkyFrame)$5);}
	| INT RULER_ SYSTEM_ coordSystem skyFrame coordSystem skyFormat
	  {FR->markerRulerSystemCmd($1, (CoordSystem)$4, (SkyFrame)$5,
	    (CoordSystem)$6, (SkyFormat)$7);}

	| INT SELECT_ {FR->markerSelectCmd($1);}
	| INT SELECT_ ONLY_ {FR->markerSelectOnlyCmd($1);}
	| INT TAG_ STRING {FR->markerTagCmd($1,$3);}
	| INT TEXT_ STRING {FR->markerTextCmd($1,$3);}
	| INT UNHIGHLITE_ {FR->markerUnhighliteCmd($1);}
	| INT UNSELECT_ {FR->markerUnselectCmd($1);}

	| INT VECTOR_ ARROW_ yesno {FR->markerVectorArrowCmd($1,$4);}
	| INT VECTOR_ POINT_ internalSystem coord numeric angle
	  {FR->markerVectorCmd($1, Vector($5), (InternalSystem)$4, $6, $7);}
	| INT VECTOR_ POINT_ coordSystem skyFrame coord 
	    coordSystem skyFormat numeric angle
	  {FR->markerVectorCmd($1, Vector($6), (CoordSystem)$4, (SkyFrame)$5,
	    $9, (CoordSystem)$7, (SkyFormat)$8, $10);}

	| INT WIDTH_ INT {FR->markerLineWidthCmd($1,$3);}

	| HIGHLITE_ ALL_ {FR->markerHighliteAllCmd();}
	| HIGHLITE_ ONLY_ numeric numeric 
	  {FR->markerHighliteOnlyCmd(Vector($3,$4));}
	| HIGHLITE_ TOGGLE_ numeric numeric
	  {FR->markerHighliteToggleCmd(Vector($3,$4));}

	| LIST_ markerList
	| LOAD_ markerLoad
	| MOVE_ markerMoveSelected
	| PRESERVE_ yesno {FR->markerPreserveCmd($2);}
	| PROPERTY_ markerProperty yesno {FR->markerPropertyCmd($2,$3);}
	| PROPERTY_ markerProperty yesno numeric numeric
	  {FR->markerPropertyCmd($2,$3,Vector($4,$5));}
	| ROTATE_ BEGIN_ numeric numeric 
	  {FR->markerRotateBeginCmd(Vector($3,$4));}
	| ROTATE_ MOTION_ numeric numeric INT
	  {FR->markerRotateMotionCmd(Vector($3,$4),$5);}
	| ROTATE_ END_ {FR->markerRotateEndCmd();}
	| SAVE_ STRING markerFormat coordSystem skyFrame skyFormat yesno yesno
	  {FR->markerSaveCmd($2, (FrameBase::MarkerFormat)$3,
	    (CoordSystem)$4, (SkyFrame)$5, (SkyFormat)$6, $7, $8);}
	| SAVE_ TEMPLATE_ STRING {FR->markerSaveTemplateCmd($3);}
	| SELECT_ markerSelect

	| SHOW_ markerShow

	| STRING COLOR_ STRING {FR->markerColorCmd($1,$3);}
	| STRING COPY_ {FR->markerCopyCmd($1);}
	| STRING DELETE_ {FR->markerDeleteCmd($1);}
	| STRING CUT_ {FR->markerCutCmd($1);}
	| STRING FONT_ STRING {FR->markerFontCmd($1,$3);}
	| STRING HIGHLITE_ {FR->markerHighliteCmd($1);}
	| STRING HIGHLITE_ ONLY_ {FR->markerHighliteOnlyCmd($1);}
	| STRING MOVE_ numeric numeric {FR->markerMoveCmd($1,Vector($3,$4));}
	| STRING MOVE_ FRONT_ {FR->markerFrontCmd($1);}
	| STRING MOVE_ BACK_ {FR->markerBackCmd($1);}
	| STRING MOVE_ TO_ internalSystem coord 
	  {FR->markerMoveToCmd($1,Vector($5),(InternalSystem)$4);}
	| STRING MOVE_ TO_ coordSystem skyFrame coord 
	  {FR->markerMoveToCmd($1,Vector($6),(CoordSystem)$4,(SkyFrame)$5);}
	| STRING PROPERTY_ markerProperty yesno 
	  {FR->markerPropertyCmd($1,$3,$4);}
	| STRING SELECT_ {FR->markerSelectCmd($1);}
	| STRING SELECT_ ONLY_ {FR->markerSelectOnlyCmd($1);}
	| STRING UNHIGHLITE_ {FR->markerUnhighliteCmd($1);}
	| STRING UNSELECT_ {FR->markerUnselectCmd($1);}

	| TAG_ EDIT_ STRING STRING {FR->markerTagEditCmd($3,$4);}
	| TAG_ DELETE_ STRING {FR->markerTagDeleteCmd($3);}
	| TAG_ DELETE_ ALL_ {FR->markerTagDeleteAllCmd();}
	| TAG_ STRING {FR->markerTagCmd($2);}
	| TAG_ UPDATE_ STRING {FR->markerTagUpdateCmd($3);}

	| PASTE_ {FR->markerPasteCmd();}
	| PASTE_ coordSystem skyFrame 
	  {FR->markerPasteCmd((CoordSystem)$2,(SkyFrame)$3);}
	| UNDO_ {FR->markerUndoCmd();}
	| UNHIGHLITE_ ALL_ {FR->markerUnhighliteAllCmd();}
	| UNSELECT_ ALL_ {FR->markerUnselectAllCmd();}
	| WIDTH_ INT {FR->markerLineWidthCmd($2);}
	;

markerCallBack : SELECT_ {$$ = CallBack::SELECTCB;}
	| UNSELECT_ {$$ = CallBack::UNSELECTCB;}
	| HIGHLITE_ {$$ = CallBack::HIGHLITECB;}
	| UNHIGHLITE_ {$$ = CallBack::UNHIGHLITECB;}
	| BEGIN_ MOVE_  {$$ = CallBack::MOVEBEGINCB;}
	| MOVE_  {$$ = CallBack::MOVECB;}
	| END_ MOVE_  {$$ = CallBack::MOVEENDCB;}
	| BEGIN_ EDIT_ {$$ = CallBack::EDITBEGINCB;}
	| EDIT_ {$$ = CallBack::EDITCB;}
	| END_ EDIT_ {$$ = CallBack::EDITENDCB;}
	| BEGIN_ ROTATE_ {$$ = CallBack::ROTATEBEGINCB;}
	| ROTATE_ {$$ = CallBack::ROTATECB;}
	| END_ ROTATE_ {$$ = CallBack::ROTATEENDCB;}
	| DELETE_ {$$ = CallBack::DELETECB;}
	| TEXT_ {$$ = CallBack::TEXTCB;}
	| COLOR_ {$$ = CallBack::COLORCB;}
	| WIDTH_ {$$ = CallBack::LINEWIDTHCB;}
	| PROPERTY_ {$$ = CallBack::PROPERTYCB;}
	| FONT_ {$$ = CallBack::FONTCB;}
	| UPDATE_ {$$ = CallBack::UPDATECB;}
	;

markerCreate : 
	  CIRCLE_ numeric numeric 
	    numeric 
	    markerProperties
	  {FR->createCircleCmd(Vector($2,$3)*FR->getCanvasToRef(),
	    $4,
	    currentColor,currentWidth,currentFont,
	    currentText,currentProps,NULL,taglist,cblist);}
	| ANNULUS_ numeric numeric 
	    numeric numeric INT 
	    markerProperties
	  {FR->createAnnulusCmd(Vector($2,$3)*FR->getCanvasToRef(),
	    $4,$5,$6,
	    currentColor,currentWidth,currentFont,
	    currentText,currentProps,NULL,taglist,cblist);}
	| CPANDA_ numeric numeric 
	    angle angle INT
	    numeric numeric INT 
	    markerProperties
	  {FR->createCpandaCmd(Vector($2,$3)*FR->getCanvasToRef(),
	    $4,$5,$6,
	    $7,$8,$9,
	    currentColor,currentWidth,currentFont,
	    currentText,currentProps,NULL,taglist,cblist);}

	| ELLIPSE_ numeric numeric 
	    numeric numeric 
	    optangle 
	    markerProperties
	  {FR->createEllipseCmd(Vector($2,$3)*FR->getCanvasToRef(),
	    Vector($4,$5),
	    $6,
	    currentColor,currentWidth,currentFont,
	    currentText,currentProps,NULL,taglist,cblist);}
	| ELLIPSEANNULUS_ numeric numeric 
	    numeric numeric numeric INT 
	    optangle 
	    markerProperties
	  {FR->createEllipseAnnulusCmd(Vector($2,$3)*FR->getCanvasToRef(),
	    Vector($4,$5), 
	    Vector($6,$6*$4/$5),$7,
	    $8,
	    currentColor,currentWidth,currentFont, 
	    currentText,currentProps,NULL,taglist,cblist);}
	| EPANDA_ numeric numeric 
	    angle angle INT
	    numeric numeric numeric INT 
	    optangle 
	    markerProperties
	  {FR->createEpandaCmd(Vector($2,$3)*FR->getCanvasToRef(),
	    $4,$5,$6,
	    Vector($7,$8),
	    Vector($9,$9*$7/$8),$10,
	    $11,
	    currentColor,currentWidth,currentFont,
	    currentText,currentProps,NULL,taglist,cblist);}

	| BOX_ numeric numeric 
	    numeric numeric 
	    optangle 
	    markerProperties
	  {FR->createBoxCmd(Vector($2,$3)*FR->getCanvasToRef(),
	    Vector($4,$5),
	    $6,
	    currentColor,currentWidth,currentFont,
	    currentText,currentProps,NULL,taglist,cblist);}
	| BOXANNULUS_ numeric numeric 
	    numeric numeric numeric INT 
	    optangle 
	    markerProperties
	  {FR->createBoxAnnulusCmd(Vector($2,$3)*FR->getCanvasToRef(),
	    Vector($4,$5),Vector($6,$6*$4/$5),$7,
	    $8,
	    currentColor,currentWidth,currentFont,
	    currentText,currentProps,NULL,taglist,cblist);}
	| BPANDA_ numeric numeric 
	    angle angle INT
	    numeric numeric numeric INT 
	    optangle 
	    markerProperties
	  {FR->createBpandaCmd(Vector($2,$3)*FR->getCanvasToRef(),
	    $4,$5,$6,
	    Vector($7,$8),
	    Vector($9,$9*$7/$8),$10,
	    $11,
	    currentColor,currentWidth,currentFont,
	    currentText,currentProps,NULL,taglist,cblist);}

	| LINE_ numeric numeric 
	    numeric numeric 
	    markerProperties
	  {FR->createLineCmd(Vector($2,$3)*FR->getCanvasToRef(),
	    Vector($4,$5)*FR->getCanvasToRef(),
	    0, 0,
	    currentColor,currentWidth,currentFont,
	    currentText,currentProps,NULL,taglist,cblist);}
	| VECTOR_ numeric numeric 
	    numeric numeric 
	    markerProperties
	  {FR->createVectCmd(Vector($2,$3)*FR->getCanvasToRef(),
	    Vector($4,$5)*FR->getCanvasToRef(),
	    1,
	    currentColor,currentWidth,currentFont,
	    currentText,currentProps,NULL,taglist,cblist);}
	| PROJECTION_ numeric numeric 
	    numeric numeric 
	    numeric 
	    markerProperties
	  {FR->createProjectionCmd(Vector($2,$3)*FR->getCanvasToRef(),
	    Vector($4,$5)*FR->getCanvasToRef(),
	    $6,
	    NULL,NULL,
	    currentColor,currentWidth,currentFont,
	    currentText,currentProps,NULL,taglist,cblist);}
	| PROJECTION_ numeric numeric 
	    numeric numeric 
	    numeric 
	    STRING STRING 
	    markerProperties
	  {FR->createProjectionCmd(Vector($2,$3)*FR->getCanvasToRef(),
	    Vector($4,$5)*FR->getCanvasToRef(),
	    $6,
	    $7,$8,
	    currentColor,currentWidth,currentFont,
	    currentText,currentProps,NULL,taglist,cblist);}
	| RULER_ numeric numeric 
	    numeric numeric 
	    coordSystem skyFrame 
	    markerProperties
	  {FR->createRulerCmd(Vector($2,$3)*FR->getCanvasToRef(),
	    Vector($4,$5)*FR->getCanvasToRef(),
	    (CoordSystem)$6, (SkyFrame)$7, PHYSICAL, DEGREES,
	    currentColor,currentWidth,currentFont,
	    currentText,currentProps,NULL,taglist,cblist);}

	| CIRCLE_ POINT_ numeric numeric 
	    markerProperties
	  {FR->createCirclePointCmd(Vector($3,$4)*FR->getCanvasToRef(),
	    currentColor,currentWidth,currentFont,
	    currentText,currentProps,NULL,taglist,cblist);}
	| BOX_ POINT_ numeric numeric 
	    markerProperties
	  {FR->createBoxPointCmd(Vector($3,$4)*FR->getCanvasToRef(),
	    currentColor,currentWidth,currentFont,
	    currentText,currentProps,NULL,taglist,cblist);}
	| DIAMOND_ POINT_ numeric numeric 
	    markerProperties
	  {FR->createDiamondPointCmd(Vector($3,$4)*FR->getCanvasToRef(),
	    currentColor,currentWidth,currentFont,
	    currentText,currentProps,NULL,taglist,cblist);}
	| CROSS_ POINT_ numeric numeric 
	    markerProperties
	  {FR->createCrossPointCmd(Vector($3,$4)*FR->getCanvasToRef(),
	    currentColor,currentWidth,currentFont,
	    currentText,currentProps,NULL,taglist,cblist);}
	| 'X' POINT_ numeric numeric 
	    markerProperties
	  {FR->createExPointCmd(Vector($3,$4)*FR->getCanvasToRef(),
	    currentColor,currentWidth,currentFont,
	    currentText,currentProps,NULL,taglist,cblist);}
	| ARROW_ POINT_ numeric numeric 
	    markerProperties
	  {FR->createArrowPointCmd(Vector($3,$4)*FR->getCanvasToRef(),
	    currentColor,currentWidth,currentFont,
	    currentText,currentProps,NULL,taglist,cblist);}
	| BOXCIRCLE_ POINT_ numeric numeric 
	    markerProperties
	  {FR->createBoxCirclePointCmd(Vector($3,$4)*FR->getCanvasToRef(),
	    currentColor,currentWidth,currentFont,
	    currentText,currentProps,NULL,taglist,cblist);}

	| TEXT_ numeric numeric 
	    optangle 
	    markerProperties
	  {FR->createTextCmd(Vector($2,$3)*FR->getCanvasToRef(),
	    $4,
	    currentColor,currentWidth,currentFont,
	    currentText,currentProps,NULL,taglist,cblist);}

	| POLYGON_ numeric numeric 
	    numeric numeric 
	    markerProperties
	  {FR->createPolygonCmd(Vector($2,$3)*FR->getCanvasToRef(),
	    Vector($4,$5),
	    currentColor,currentWidth,currentFont,
	    currentText,currentProps,NULL,taglist,cblist);}

	| COMPASS_ numeric numeric 
	    numeric 
	    coordSystem skyFrame
	    markerProperties
	  {FR->createCompassCmd(Vector($2,$3)*FR->getCanvasToRef(), 
	    $4,
	    "N", "E", 1, 1,
	    (CoordSystem)$5, (SkyFrame)$6,
	    currentColor,currentWidth,currentFont,
	    currentText,currentProps,NULL,taglist,cblist);}

	| COMPOSITE_ markerProperties
	  {FR->createCompositeCmd(
	    currentColor,currentWidth,currentFont,
	    currentText,currentProps,NULL,taglist,cblist);}

	| TEMPLATE_ numeric numeric STRING
	  {FR->createTemplateCmd(Vector($2,$3)*FR->getCanvasToRef(), $4);}

	| TEMPLATE_ numeric numeric VAR_ STRING
	  {FR->createTemplateVarCmd(Vector($2,$3)*FR->getCanvasToRef(), $5);}
	;

markerEdit : BEGIN_ numeric numeric INT 
	  {FR->markerEditBeginCmd(Vector($2,$3),$4);}
	| MOTION_ numeric numeric INT 
	  {FR->markerEditMotionCmd(Vector($2,$3),$4);}
	| END_ {FR->markerEditEndCmd();}
	;

markerFormat : DS9_ {$$ = FrameBase::DS9;}
	| CIAO_ {$$ = FrameBase::CIAO;}
	| SAOTNG_ {$$ = FrameBase::SAOTNG;}
	| SAOIMAGE_ {$$ = FrameBase::SAOIMAGE;}
	| PROS_ {$$ = FrameBase::PROS;}
	| XY_ {$$ = FrameBase::RAWXY;}
	;

markerGet : COLOR_ {FR->getMarkerColorCmd();}
	| FONT_ {FR->getMarkerFontCmd();}
	| HANDLE_ numeric numeric {FR->getMarkerHandleCmd(Vector($2,$3));}
	| ID_ numeric numeric {FR->getMarkerIdCmd(Vector($2,$3));}
	| ID_ ALL_ {FR->getMarkerIdAllCmd();}
	| INT ANGLE_ {FR->getMarkerAngleCmd($1);}
	| INT ANGLE_ internalSystem  {FR->getMarkerAngleCmd($1);}
	| INT ANGLE_ coordSystem skyFrame
	  {FR->getMarkerAngleCmd($1,(CoordSystem)$3, (SkyFrame)$4);}
	| INT ANNULUS_ RADIUS_ internalSystem
	  {FR->getMarkerAnnulusRadiusCmd($1, (InternalSystem)$4);}
	| INT ANNULUS_ RADIUS_ coordSystem skyFormat
	  {FR->getMarkerAnnulusRadiusCmd($1, (CoordSystem)$4, (SkyFormat)$5);}
	| INT BOXANNULUS_ RADIUS_ internalSystem
	  {FR->getMarkerBoxAnnulusRadiusCmd($1,(InternalSystem)$4);}
	| INT BOXANNULUS_ RADIUS_ coordSystem skyFormat
	  {FR->getMarkerBoxAnnulusRadiusCmd($1,(CoordSystem)$4,(SkyFormat)$5);}
	| INT BOX_ RADIUS_ internalSystem
	  {FR->getMarkerBoxRadiusCmd($1,(InternalSystem)$4);}
	| INT BOX_ RADIUS_ coordSystem skyFormat
	  {FR->getMarkerBoxRadiusCmd($1,(CoordSystem)$4, (SkyFormat)$5);}
	| INT BPANDA_ ANGLE_ {FR->getMarkerBpandaAnglesCmd($1);}
	| INT BPANDA_ ANGLE_ internalSystem {FR->getMarkerBpandaAnglesCmd($1);}
	| INT BPANDA_ ANGLE_ coordSystem skyFrame
	  {FR->getMarkerBpandaAnglesCmd($1, (CoordSystem)$4, (SkyFrame)$5);}
	| INT BPANDA_ RADIUS_ internalSystem
	  {FR->getMarkerBpandaRadiusCmd($1, (InternalSystem)$4);}
	| INT BPANDA_ RADIUS_ coordSystem skyFormat
	  {FR->getMarkerBpandaRadiusCmd($1, (CoordSystem)$4, (SkyFormat)$5);}
	| INT CENTER_ internalSystem
	  {FR->getMarkerCenterCmd($1, (InternalSystem)$3);}
	| INT CENTER_ coordSystem skyFrame skyFormat 
	  {FR->getMarkerCenterCmd($1, (CoordSystem)$3, (SkyFrame)$4, 
	    (SkyFormat)$5);}
	| INT CIRCLE_ RADIUS_ internalSystem
	  {FR->getMarkerCircleRadiusCmd($1, (InternalSystem)$4);}
	| INT CIRCLE_ RADIUS_ coordSystem skyFormat
	  {FR->getMarkerCircleRadiusCmd($1, (CoordSystem)$4, (SkyFormat)$5);}
	| INT COLOR_ {FR->getMarkerColorCmd($1);}
	| INT COMPASS_ ARROW_ {FR->getMarkerCompassArrowCmd($1);}
	| INT COMPASS_ LABEL_ {FR->getMarkerCompassLabelCmd($1);}
	| INT COMPASS_ RADIUS_ internalSystem
	  {FR->getMarkerCompassRadiusCmd($1, (InternalSystem)$4);}
	| INT COMPASS_ RADIUS_ coordSystem skyFormat
	  {FR->getMarkerCompassRadiusCmd($1, (CoordSystem)$4, (SkyFormat)$5);}
	| INT COMPASS_ SYSTEM_ {FR->getMarkerCompassSystemCmd($1);}
	| INT CPANDA_ ANGLE_ {FR->getMarkerCpandaAnglesCmd($1);}
	| INT CPANDA_ ANGLE_ internalSystem {FR->getMarkerCpandaAnglesCmd($1);}
	| INT CPANDA_ ANGLE_ coordSystem skyFrame
	  {FR->getMarkerCpandaAnglesCmd($1, (CoordSystem)$4, (SkyFrame)$5);}
	| INT CPANDA_ RADIUS_ internalSystem
	  {FR->getMarkerCpandaRadiusCmd($1, (InternalSystem)$4);}
	| INT CPANDA_ RADIUS_ coordSystem skyFormat
	  {FR->getMarkerCpandaRadiusCmd($1, (CoordSystem)$4, (SkyFormat)$5);}
	| INT ELLIPSE_ RADIUS_ internalSystem
	  {FR->getMarkerEllipseRadiusCmd($1, (InternalSystem)$4);}
	| INT ELLIPSE_ RADIUS_ coordSystem skyFormat
	  {FR->getMarkerEllipseRadiusCmd($1, (CoordSystem)$4, (SkyFormat)$5);}
	| INT ELLIPSEANNULUS_ RADIUS_ internalSystem
	  {FR->getMarkerEllipseAnnulusRadiusCmd($1, (InternalSystem)$4);}
	| INT ELLIPSEANNULUS_ RADIUS_ coordSystem skyFormat
	  {FR->getMarkerEllipseAnnulusRadiusCmd($1, (CoordSystem)$4, 
	    (SkyFormat)$5);}
	| INT EPANDA_ ANGLE_ {FR->getMarkerEpandaAnglesCmd($1);}
	| INT EPANDA_ ANGLE_ internalSystem {FR->getMarkerEpandaAnglesCmd($1);}
	| INT EPANDA_ ANGLE_ coordSystem skyFrame
	  {FR->getMarkerEpandaAnglesCmd($1, (CoordSystem)$4, (SkyFrame)$5);}
	| INT EPANDA_ RADIUS_ internalSystem
	  {FR->getMarkerEpandaRadiusCmd($1, (InternalSystem)$4);}
	| INT EPANDA_ RADIUS_ coordSystem skyFormat
	  {FR->getMarkerEpandaRadiusCmd($1, (CoordSystem)$4, (SkyFormat)$5);}
	| INT FONT_ {FR->getMarkerFontCmd($1);}
	| INT LINE_ ARROW_ {FR->getMarkerLineArrowCmd($1);}
	| INT LINE_ LENGTH_ coordSystem skyFormat
	  {FR->getMarkerLineLengthCmd($1, (CoordSystem)$4, (SkyFormat)$5);}
	| INT LINE_ POINT_ internalSystem
	  {FR->getMarkerLineCmd($1, (InternalSystem)$4);}
	| INT LINE_ POINT_ coordSystem skyFrame skyFormat
	  {FR->getMarkerLineCmd($1, (CoordSystem)$4,
	    (SkyFrame)$5, (SkyFormat)$6);}
	| INT MAP_ LENGTH_ numeric coordSystem skyFormat
	  {FR->getMarkerMapLenFromRefCmd($1, $4,
	    (CoordSystem)$5,(SkyFormat)$6);}
	| INT PROJECTION_ STRING STRING STRING STRING
	  {FR->getMarkerProjectionCmd($1,$3,$4,$5,$6);}
	| INT PROJECTION_ POINT_ internalSystem
	  {FR->getMarkerProjectionPointsCmd($1, (InternalSystem)$4);}
	| INT PROJECTION_ POINT_ coordSystem skyFrame skyFormat
	  {FR->getMarkerProjectionPointsCmd($1, (CoordSystem)$4,
	    (SkyFrame)$5, (SkyFormat)$6);}
	| INT PROJECTION_ LENGTH_ coordSystem skyFormat
	  {FR->getMarkerProjectionLengthCmd($1, (CoordSystem)$4, 
	    (SkyFormat)$5);}
	| INT PROJECTION_ METHOD_ {FR->getMarkerProjectionMethodCmd($1);} 
	| INT PROJECTION_ THICK_ internalSystem
	  {FR->getMarkerProjectionWidthCmd($1, (InternalSystem)$4);}
	| INT PROJECTION_ THICK_ coordSystem skyFormat
	  {FR->getMarkerProjectionWidthCmd($1,(CoordSystem)$4,(SkyFormat)$5);}
	| INT PROJECTION_ SYSTEM_ {FR->getMarkerProjectionSystemCmd($1);}
	| INT PROPERTY_ {FR->getMarkerPropertyCmd($1);}
	| INT PROPERTY_ markerProperty {FR->getMarkerPropertyCmd($1,$3);}
	| INT RULER_ LENGTH_ coordSystem skyFormat
	  {FR->getMarkerRulerLengthCmd($1,(CoordSystem)$4,(SkyFormat)$5);}
	| INT RULER_ POINT_ internalSystem
	  {FR->getMarkerRulerPointCmd($1, (InternalSystem)$4);}
	| INT RULER_ POINT_ coordSystem skyFrame skyFormat
	  {FR->getMarkerRulerPointCmd($1, (CoordSystem)$4,
	    (SkyFrame)$5, (SkyFormat)$6);}
	| INT RULER_ SYSTEM_ {FR->getMarkerRulerSystemCmd($1);}
	| INT TAG_ {FR->getMarkerTagCmd($1);}
	| INT TAG_ INT {FR->getMarkerTagCmd($1,$3);}
	| INT TEXT_ {FR->getMarkerTextCmd($1);}
	| INT TYPE_ {FR->getMarkerTypeCmd($1);}
	| INT VECTOR_ ARROW_ {FR->getMarkerVectorArrowCmd($1);}
	| INT VECTOR_ LENGTH_ coordSystem skyFormat
	  {FR->getMarkerVectorLengthCmd($1, (CoordSystem)$4, (SkyFormat)$5);}
	| INT VECTOR_ POINT_ internalSystem
	  {FR->getMarkerVectorCmd($1, (InternalSystem)$4);}
	| INT VECTOR_ POINT_ coordSystem skyFrame skyFormat
	  {FR->getMarkerVectorCmd($1, (CoordSystem)$4,
	    (SkyFrame)$5, (SkyFormat)$6);}
	| INT WIDTH_ {FR->getMarkerLineWidthCmd($1);}
	| HIGHLITE_ markerGetHighlite
	| HIGHLITE_ NUMBER_ {FR->getMarkerHighlitedNumberCmd();}
	| POLYGON_ SEGMENT_ numeric numeric
	  {FR->getMarkerPolygonSegmentCmd(Vector($3,$4));}
	| PRESERVE_ {FR->getMarkerPreserveCmd();}
	| PROPERTY_ markerProperty {FR->getMarkerPropertyCmd($2);}
	| SELECT_ markerGetSelect
	| SELECT_ NUMBER_ {FR->getMarkerSelectedNumberCmd();}

	| SHOW_ markerGetShow

	| STRING COLOR_ {FR->getMarkerColorCmd($1);}
	| STRING FONT_ {FR->getMarkerFontCmd($1);}
	| STRING PROPERTY_ markerProperty {FR->getMarkerPropertyCmd($1,$3);}
	| STRING TAG_ {FR->getMarkerTagCmd($1);}
	| STRING TAG_ NUMBER_ {FR->getMarkerTagNumberCmd($1);}

	| TAG_ ALL_ {FR->getMarkerTagsCmd();}
	| TAG_ DEFAULT_ NAME_ {FR->getMarkerTagDefaultNameCmd();}

	| WIDTH_ {FR->getMarkerLineWidthCmd();}
	;

markerGetHighlite : /* empty */ {FR->getMarkerHighlitedCmd();}
	| numeric numeric {FR->getMarkerHighlitedCmd(Vector($1,$2));}
	;

markerGetSelect : /* empty */ {FR->getMarkerSelectedCmd();}
	| numeric numeric {FR->getMarkerSelectedCmd(Vector($1,$2));}
	;

markerGetShow : /* empty */ {FR->getMarkerShowCmd();}
	| TEXT_ {FR->getMarkerShowTextCmd();}
	;

markerInitProp : {
	  strncpy(currentFont, "helvetica 10 normal", 32);
	  strncpy(currentColor, "green", 16);
	  currentWidth = 1;
	  strncpy(currentText, "", 80);
	  currentProps = defaultProps;
	  taglist.deleteAll();
	  cblist.deleteAll();
	}
	;

markerLayer : /* empty */ {FR->markerLayerCmd(FrameBase::USER);}
	| USER_ {FR->markerLayerCmd(FrameBase::USER);}
	| CATALOG_  {FR->markerLayerCmd(FrameBase::CATALOG);}
	;

markerList : markerFormat coordSystem skyFrame skyFormat 
	    yesno yesno markerQuery markerTags
	  {FR->markerListCmd((FrameBase::MarkerFormat)$1,
	    (CoordSystem)$2, (SkyFrame)$3, (SkyFormat)$4, $5, $6,
	    0, propQMask, propQValue, taglist);}
	| SELECT_ markerFormat coordSystem skyFrame skyFormat 
	    yesno yesno markerQuery markerTags
	  {FR->markerListCmd((FrameBase::MarkerFormat)$2, 
	    (CoordSystem)$3, (SkyFrame)$4, (SkyFormat)$5, $6, $7,
	    1, propQMask, propQValue, taglist);}
	;

markerLoad : markerFormat STRING
	  {FR->markerLoadCmd((FrameBase::MarkerFormat)$1,$2);}
	| markerFormat STRING coordSystem skyFrame
	  {FR->markerLoadCmd((FrameBase::MarkerFormat)$1,$2,
	    (CoordSystem)$3,(SkyFrame)$4);}
	| markerFormat INT
	  {FR->markerLoadCmd((FrameBase::MarkerFormat)$1,$2);}
	| markerFormat INT coordSystem skyFrame
	  {FR->markerLoadCmd((FrameBase::MarkerFormat)$1,$2,
	    (CoordSystem)$3,(SkyFrame)$4);}
	| FITS_ STRING STRING INT STRING 
	  {FR->markerLoadFitsCmd($2, $3, $4, $5);}
	;

markerMoveSelected : numeric numeric {FR->markerMoveCmd(Vector($1,$2));}
	| FRONT_ {FR->markerFrontCmd();}
	| BACK_ {FR->markerBackCmd();}
	| BEGIN_ numeric numeric {FR->markerMoveBeginCmd(Vector($2,$3));}
	| MOTION_ numeric numeric {FR->markerMoveMotionCmd(Vector($2,$3));}
	| END_ {FR->markerMoveEndCmd();}
	| TO_ internalSystem coord 
	  {FR->markerMoveToCmd(Vector($3), (InternalSystem)$2);}
	| TO_ coordSystem skyFrame coord 
	  {FR->markerMoveToCmd(Vector($4), (CoordSystem)$2, (SkyFrame)$3);}
	;

markerProps : markerProps markerProp
	| markerProp
	;

markerProp : markerProperty '=' yesno {setProps(&currentProps,$1,$3);}
	| COLOR_ '=' STRING {strncpy(currentColor,$3,16);}
	| WIDTH_ '=' INT {currentWidth = $3;}
	| FONT_ '=' STRING {strncpy(currentFont,$3,32);}
	| TEXT_ '=' STRING {strncpy(currentText,$3,80);}
	| tag
	| callback
	;

markerProperty : NONE_ {$$ = Marker::NONE;}
	| SELECT_ {$$ = Marker::SELECT;}
	| HIGHLITE_ {$$ = Marker::HIGHLITE;}
	| EDIT_ {$$ = Marker::EDIT;}
	| MOVE_  {$$ = Marker::MOVE;}
	| ROTATE_ {$$ = Marker::ROTATE;}
	| DELETE_ {$$ = Marker::DELETE;}
	| INCLUDE_ {$$ = Marker::INCLUDE;}
	| SOURCE_ {$$ = Marker::SOURCE;}
	| FIXED_ {$$ = Marker::FIXED;}
	;

markerProperties : /* empty */ markerInitProp
	| markerInitProp markerProps
	;

markerQuery: /* empty */ {propQMask=0;propQValue=0;}
	| {propQMask=0;propQValue=0;} queries
	;

markerSelect : ALL_ {FR->markerSelectAllCmd();}
	| ONLY_ numeric numeric {FR->markerSelectOnlyCmd(Vector($2,$3));}
	| TOGGLE_ {FR->markerSelectToggleCmd();}
	| TOGGLE_ numeric numeric {FR->markerSelectToggleCmd(Vector($2,$3));}
	;

markerShow : INT {FR->markerShowCmd($1);}
	| TEXT_ INT {FR->markerShowTextCmd($2);}
	;

queries : queries query
	| query
	;

query	: markerProperty '=' yesno 
	  {propQMask |= $1; setProps(&propQValue,$1,$3);}
	;

markerTags: /* empty */ {taglist.deleteAll()}
	| {taglist.deleteAll()} tags
	;

tags	: tags tag
	| tag
	;

tag	: TAG_ '=' STRING {taglist.append(new Tag($3));}
	;

callback : CALLBACK_ '=' markerCallBack STRING STRING {cblist.append(
	new CallBack(FR->getInterp(),(CallBack::Type)$3,$4,$5));}
	;

mask	: COLOR_ STRING {FR->maskColorCmd($2);}
	| INT {FR->maskCmd($1);}
	;

orient	: 'X' {FR->orientCmd(XX);}
	| 'Y' {FR->orientCmd(YY);}
	| XY_ {FR->orientCmd(XY);}
	| NONE_ {FR->orientCmd(NORMAL);}
	;

pan	: numeric numeric numeric numeric 
	  {FR->panCmd(Vector($1,$2),Vector($3,$4));}
	| internalSystem numeric numeric
	    {FR->panCmd(Vector($2,$3), (InternalSystem)$1);}
	| coordSystem skyFrame coord
	    {FR->panCmd(Vector($3), (CoordSystem)$1, (SkyFrame)$2);}
	| TO_ panTo
	| BBOX_ numeric numeric {FR->panBBoxCmd(Vector($2,$3));}
	| MOTION_ panMotion
	| PRESERVE_ yesno {FR->panPreserveCmd($2);}
	;

panTo	: internalSystem numeric numeric
	    {FR->panToCmd(Vector($2,$3), (InternalSystem)$1);}
	| coordSystem skyFrame coord
	    {FR->panToCmd(Vector($3), (CoordSystem)$1, (SkyFrame)$2);}
	;

panMotion : BEGIN_ numeric numeric {FR->panMotionBeginCmd(Vector($2,$3));}
	| numeric numeric {FR->panMotionCmd(Vector($1,$2));}
	| END_ numeric numeric {FR->panMotionEndCmd(Vector($2,$3));}
	;

panner	: yesno {FR->pannerCmd($1);}
	| WCS_ wcsSystem skyFrame 
	  {FR->pannerCmd((CoordSystem)$2,(SkyFrame)$3);}
	| STRING INT INT {FR->pannerCmd($1, $2, $3);}
	| UPDATE_ {FR->updatePannerCmd();}
	;

postscript : COLORSPACE_ pscolorspace 
	  {FR->psColorSpaceCmd((Widget::PSColorSpace)$2);}
	| LEVEL_ INT {FR->psLevelCmd($2);}
	| RESOLUTION_ INT {FR->psResolutionCmd($2);}
	;

pscolorspace : BW_ {$$ = Widget::BW;}
	| GRAY_ {$$ = Widget::GRAY;}
	| RGB_ {$$ = Widget::RGB;}
	| CMYK_ {$$ = Widget::CMYK;}
	;

region	: HIGHLITE_ regionHighlite
	| SELECT_ regionSelect
	;

regionHighlite : BEGIN_ numeric numeric 
	  {FR->regionHighliteBeginCmd(Vector($2,$3));}
	| MOTION_ numeric numeric 
	  {FR->regionHighliteMotionCmd(Vector($2,$3));}
	| END_ {FR->regionHighliteEndCmd();}
	| SHIFT_ END_ {FR->regionHighliteShiftEndCmd();}
	;

regionSelect : BEGIN_ numeric numeric 
	  {FR->regionSelectBeginCmd(Vector($2,$3));}
	| MOTION_ numeric numeric {FR->regionSelectMotionCmd(Vector($2,$3));}
	| END_ {FR->regionSelectEndCmd();}
	| SHIFT_ END_ {FR->regionSelectShiftEndCmd();}
	;

rgb	: CHANNEL_ STRING {FR->setRGBChannelCmd($2);}
	| SYSTEM_ coordSystem {FR->setRGBSystemCmd((CoordSystem)$2);}
	| VIEW_ INT INT INT {FR->setRGBViewCmd($2,$3,$4);}
	;

rotate	: numeric {FR->rotateCmd(degToRad($1));}
	| numeric DEGREES_ {FR->rotateCmd(degToRad($1));}
	| MOTION_ rotateMotion
	| TO_ numeric {FR->rotateToCmd(degToRad($2));}
	| TO_ numeric DEGREES_ {FR->rotateToCmd(degToRad($2));}
	;

rotateMotion : BEGIN_ {FR->rotateMotionBeginCmd();}
	| numeric {FR->rotateMotionCmd(degToRad($1));}
	| END_ {FR->rotateMotionEndCmd();}
	;

save	: ARRAY_ saveArray
	| FITS_ saveFits
	;

saveArray : FILE_ STRING {FR->saveArrayFileCmd($2);}
	| CHANNEL_ STRING {FR->saveArrayChannelCmd($2);}
	| SOCKET_ INT {FR->saveArraySocketCmd($2);}
	;

saveFits: IMAGE_ saveFitsImage
	| TABLE_ saveFitsTable
	| RESAMPLE_ saveFitsResample
	;

saveFitsImage : FILE_ STRING compressType {FR->saveFitsImageFileCmd($2, $3);}
	| CHANNEL_ STRING compressType {FR->saveFitsImageChannelCmd($2, $3);}
	| SOCKET_ INT compressType {FR->saveFitsImageSocketCmd($2, $3);}
	;

saveFitsTable : FILE_ STRING compressType {FR->saveFitsTableFileCmd($2, $3);}
	| CHANNEL_ STRING compressType {FR->saveFitsTableChannelCmd($2, $3);}
	| SOCKET_ INT compressType {FR->saveFitsTableSocketCmd($2, $3);}
	;

saveFitsResample : FILE_ STRING compressType 
	  {FR->saveFitsResampleFileCmd($2, $3);}
	| CHANNEL_ STRING compressType 
	  {FR->saveFitsResampleChannelCmd($2, $3);}
	| SOCKET_ INT compressType 
	  {FR->saveFitsResampleSocketCmd($2, $3);}
	;

smooth	: smoothFunction INT 
	  {FR->smoothCmd((FrameBase::SmoothFunction)$1,$2);}
	| DELETE_ {FR->smoothDeleteCmd();}
	;

smoothFunction : BOXCAR_ {$$ = FrameBase::BOXCAR;}
	| GAUSSIAN_ {$$ = FrameBase::GAUSSIAN;}
	| TOPHAT_ {$$ = FrameBase::TOPHAT;}
	;

update	: /* empty */ {FR->updateFitsCmd(0);}
	| INT numeric numeric numeric numeric
	  {FR->updateFitsCmd($1,BBox($2,$3,$4,$5),0);}
	| NOW_ {FR->updateFitsCmd(1);}
	| NOW_ INT numeric numeric numeric numeric
	  {FR->updateFitsCmd($2,BBox($3,$4,$5,$6),1);}
	| FITS_ SLICE_ INT {FR->sliceCmd($3);}
	;

warp	: /* empty */ numeric numeric {FR->warpCmd(Vector($1,$2));}
	| TO_ numeric numeric {FR->warpToCmd(Vector($2,$3));}
	;

wcs	: ALIGN_ wcsAlign
	| RESET_ {FR->wcsResetCmd();}
	| REPLACE_ INT {FR->wcsReplaceCmd($2);}
	| REPLACE_ STRING {FR->wcsReplaceCmd($2);}
	| APPEND_ INT {FR->wcsAppendCmd($2);}
	| APPEND_ STRING {FR->wcsAppendCmd($2);}
	;

wcsAlign : INT wcsSystem skyFrame 
	  {FR->wcsAlignCmd($1, (CoordSystem)$2, (SkyFrame)$3);}
	| INT wcsSystem skyFrame POINTER
	  {FR->wcsAlignCmd($1, (CoordSystem)$2, (SkyFrame)$3, (FitsImage*)$4);}
	;

zoom	: numeric numeric {FR->zoomCmd(Vector($1,$2));}
	| numeric numeric ABOUT_ internalSystem numeric numeric
	  {FR->zoomAboutCmd(Vector($1,$2),Vector($5,$6),(InternalSystem)$4);}
	| numeric numeric ABOUT_ coordSystem skyFrame coord
	  {FR->zoomAboutCmd(Vector($1,$2),Vector($6),
	    (CoordSystem)$4,(SkyFrame)$5);}
	| TO_ zoomTo
	;

zoomTo	: FIT_	{FR->zoomToFitCmd();}
	| FIT_	numeric {FR->zoomToFitCmd($2);}
	| numeric numeric {FR->zoomToCmd(Vector($1,$2));}
	| numeric numeric ABOUT_ internalSystem numeric numeric
	  {FR->zoomToAboutCmd(Vector($1,$2),Vector($5,$6),
	    (InternalSystem)$4);}
	| numeric numeric ABOUT_ coordSystem skyFrame coord
	  {FR->zoomToAboutCmd(Vector($1,$2), Vector($6),
	    (CoordSystem)$4, (SkyFrame)$5);}
	;

%%

static void setProps(unsigned short* props, unsigned short prop, int value)
{
  if (value)
    *props |= prop;
  else
    *props &= ~prop;
}
