#  Copyright (C) 1999-2007
#  Smithsonian Astrophysical Observatory, Cambridge, MA, USA
#  For conditions of distribution and use, see copyright notice in "copyright"

package provide DS9 1.0

proc CreateButtons {} {
    global ds9
    global buttons

    set ds9(buttons) [frame $ds9(main).buttons]

    frame $ds9(buttons).major
    pack $ds9(buttons).major -side top -fill x -expand true

    lappend buttons(major) \
	[RadioButton $ds9(buttons).major.file [string tolower [msgcat::mc {File}]] \
	     buttons(majorCurrent) $ds9(buttons).file MajorButton]
    lappend buttons(major) \
	[RadioButton $ds9(buttons).major.edit [string tolower [msgcat::mc {Edit}]] \
	     buttons(majorCurrent) $ds9(buttons).edit MajorButton]
    lappend buttons(major) \
	[RadioButton $ds9(buttons).major.view [string tolower [msgcat::mc {View}]] \
	     buttons(majorCurrent) $ds9(buttons).view MajorButton]
    lappend buttons(major) \
	[RadioButton $ds9(buttons).major.frame [string tolower [msgcat::mc {Frame}]] \
	     buttons(majorCurrent) $ds9(buttons).frame MajorButton]
    lappend buttons(major) \
	[RadioButton $ds9(buttons).major.bin [string tolower [msgcat::mc {Bin}]] \
	     buttons(majorCurrent) $ds9(buttons).bin MajorButton]
    lappend buttons(major) \
	[RadioButton $ds9(buttons).major.zoom [string tolower [msgcat::mc {Zoom}]] \
	     buttons(majorCurrent) $ds9(buttons).zoom MajorButton]
    lappend buttons(major) \
	[RadioButton $ds9(buttons).major.scale [string tolower [msgcat::mc {Scale}]] \
	     buttons(majorCurrent) $ds9(buttons).scale1 MajorButton]
    lappend buttons(major) \
	[RadioButton $ds9(buttons).major.color1 [string tolower [msgcat::mc {Color}]] \
	     buttons(majorCurrent) $ds9(buttons).color1 MajorButton]
    lappend buttons(major) \
	[RadioButton $ds9(buttons).major.region [string tolower [msgcat::mc {Region}]] \
	     buttons(majorCurrent) $ds9(buttons).region1 MajorButton]
    lappend buttons(major) \
	[RadioButton $ds9(buttons).major.wcs1 [string tolower [msgcat::mc {WCS}]] \
	     buttons(majorCurrent) $ds9(buttons).wcs1 MajorButton]
    lappend buttons(major) \
	[RadioButton $ds9(buttons).major.help [string tolower [msgcat::mc {Help}]] \
	     buttons(majorCurrent) $ds9(buttons).help MajorButton]

    FileButtons
    EditButtons
    ViewButtons
    FrameButtons
    BinButtons
    ZoomButtons
    ScaleButtons
    ColorButtons
    RegionButtons
    WCSButtons
    HelpButtons

    set buttons(majorPrev) {}
    set buttons(majorCurrent) $ds9(buttons).file

    $ds9(buttons).major.file invoke
}

proc ButtonButton {button text cmd} {
    button $button \
	-text $text \
	-command $cmd \
	-takefocus 0 \
	-bd 1 \
	-height 1 \
	-padx 2 -pady 2

    pack $button -side left -fill x -expand true
}

proc RadioButton {button text var value cmd} {
    global ds9
    switch $ds9(wm) {
	x11 -
	win32 {
	    radiobutton $button \
		-text $text \
		-variable $var \
		-value $value \
		-command $cmd \
		-indicatoron false \
		-selectcolor {} \
		-takefocus 0 \
		-bd 1 \
		-height 1 \
		-padx 2 -pady 3
	}
	aqua {
	    button $button \
		-text $text \
		-command \
		"RadioButtonAqua $button $var $value $cmd buttons(major)" \
		-takefocus 0 \
		-bd 1 \
		-height 1 \
		-padx 2 -pady 2
	}
    }

    pack $button -side left -fill x -expand true
    return $button
}

proc RadioButtonAqua {button var value cmd list} {
    global buttons

    foreach b [expr $$list] {
	$b configure -state normal
    } 
    $button configure -state active

    set $var $value
    eval "$cmd"
}

proc CheckButton {button text var cmd} {
    global ds9
    switch $ds9(wm) {
	x11 -
	win32 {
	    checkbutton $button \
		-text $text \
		-variable $var \
		-command $cmd \
		-indicatoron false \
		-selectcolor {} \
		-takefocus 0 \
		-bd 1 \
		-height 1 \
		-padx 2 -pady 3
	}
	aqua {
	    button $button \
		-text $text \
		-command "CheckButtonAqua $button $var $cmd" \
		-takefocus 0 \
		-bd 1 \
		-height 1 \
		-padx 2 -pady 2
	}
    }

    pack $button -side left -fill x -expand true
}

proc CheckButtonAqua {button varname cmd} {
    upvar #0 $varname var
    global buttons

    set $varname [expr !$var]
    if {$var} {
	$button configure -state active
    } else {
	$button configure -state normal
    }
    eval "$cmd"
}

proc MajorButton {} {
    global buttons

    pack forget $buttons(majorPrev)
    pack $buttons(majorCurrent) -side top -fill x -expand true
    set buttons(majorPrev) $buttons(majorCurrent)
}



