#  Copyright (C) 1999-2007
#  Smithsonian Astrophysical Observatory, Cambridge, MA, USA
#  For conditions of distribution and use, see copyright notice in "copyright"

package provide DS9 1.0

proc DSSESODialog {} {
    global dsseso

    if [winfo exists $dsseso(top)] {
	raise $dsseso(top)
	return
    }

    set varname dssesoi
    upvar #0 $varname var
    global $varname

    set var(top) $dsseso(top)
    set var(mb) $dsseso(mb)
    set var(system) $dsseso(system)
    set var(sky) $dsseso(sky)
    set var(skyformat) $dsseso(skyformat)
    set var(rformat) $dsseso(rformat)
    set var(width) $dsseso(width)
    set var(height) $dsseso(height)
    set var(survey) $dsseso(survey)
    set var(mode) $dsseso(mode)
    set var(save) $dsseso(save)
    set var(valid) $dsseso(valid)

    set w $var(top)
    IMGSVRInit $varname "DSS-ESO [msgcat::mc {Server}]" DSSESOExec DSSESOAck

    # menu
    IMGSVRFileMenu $varname
    AREditMenu $varname
    NSVRServerMenu $varname
    ARCoordMenu $varname
    ARSizeMenu $varname

    $var(mb) add cascade -label Survey -menu $var(mb).survey
    menu $var(mb).survey
    $var(mb).survey add radiobutton -label {DSS1} \
	-variable ${varname}(survey) -value dss
    $var(mb).survey add radiobutton -label {DSS2-red} \
	-variable ${varname}(survey) -value dss2red
    $var(mb).survey add radiobutton -label {DSS2-blue} \
	-variable ${varname}(survey) -value dss2blue

    IMGSVRPrefsMenu $varname
    IMGSVRUpdate $varname 1
}

proc DSSESOExec {varname} {
    upvar #0 $varname var
    global $varname

    if {$var(save)} {
	set mime "application/x-fits"
	set var(fn) [SaveFileDialog dssesofbox]
	if {$var(fn) == {}} {
	    return
	}
    } else {
	set mime "display/gz-fits"
	set var(fn) [tmpnam dss ".fits.gz"]
    }

    # size - change to arcmin
    switch -- $var(rformat) {
	degrees {
	    set ww [expr $var(width)*60.]
	    set hh [expr $var(height)*60.]
	}
	arcmin {
	    set ww $var(width)
	    set hh $var(height)
	}
	arcsec {
	    set ww [expr $var(width)/60.]
	    set hh [expr $var(height)/60.]
	}
    }
    if {$ww>60} {
	set ww 60
    }
    if {$hh>60} {
	set hh 60
    }

    # query
    set query [http::formatQuery ra $var(x) dec $var(y) equinox J2000\
			x $ww y $hh \
			mime-type "$mime" \
			Sky-Survey $var(survey)]
    set var(query) {}
    # Load image
    # we can't use -query because eso needs a GET not a POST
    set var(url) "http://archive.eso.org/dss/dss?$query"
    IMGSVRLoad $varname
}

proc DSSESOAck {varname} {
    upvar #0 $varname var
    global $varname

set msg {Acknowledgments for the DSSESO 

The Digitized Sky Surveys were produced at the Space Telescope Science
Institute under U.S.  Government grant NAG W-2166. The images of these
surveys are based on photographic data obtained using the Oschin
Schmidt Telescope on Palomar Mountain and the UK Schmidt Telescope.
The plates were processed into the present compressed digital form
with the permission of these institutions.

The National Geographic Society - Palomar Observatory Sky Atlas
(POSS-I) was made by the California Institute of Technology with
grants from the National Geographic Society.

The Second Palomar Observatory Sky Survey (POSS-II) was made by the
California Institute of Technology with funds from the National
Science Foundation, the National Geographic Society, the Sloan
Foundation, the Samuel Oschin Foundation, and the Eastman Kodak
Corporation.

The Oschin Schmidt Telescope is operated by the California Institute
of Technology and Palomar Observatory.

The UK Schmidt Telescope was operated by the Royal Observatory
Edinburgh, with funding from the UK Science and Engineering Research
Council (later the UK Particle Physics and Astronomy Research
Council), until 1988 June, and thereafter by the Anglo-Australian
Observatory. The blue plates of the southern Sky Atlas and its
Equatorial Extension (together known as the SERC-J), as well as the
Equatorial Red (ER), and the Second Epoch [red] Survey (SES) were all
taken with the UK Schmidt.
}

    SimpleTextDialog dssesoack Acknowledgment 80 40 insert top $msg
}

# Process Cmds

proc ProcessDSSESOCmd {varname iname flag fcName} {
    upvar $varname var
    upvar $iname i
    upvar $fcName fc

    global current

    set vvarname dssesoi
    upvar #0 $vvarname vvar
    global $vvarname

    # do we have a rgb frame
    if {[$current(frame) get type] == {rgb}} {
	set base 0
    } else {
	set base 1
    }

    DSSESODialog

    set vvar(mode) current

    switch -- [string tolower [lindex $var $i]] {
	survey {
	    incr i
	    set vvar(survey) [lindex $var $i]
	}
	size {
	    incr i
	    set vvar(width) [lindex $var $i]
	    incr i
	    set vvar(height) [lindex $var $i]
	}
	width {
	    incr i
	    set vvar(width) [lindex $var $i]
	}
	height {
	    incr i
	    set vvar(height) [lindex $var $i]
	}
	coord {
	    if {$flag} {
		MultiLoad fc
	    }
	    incr i
	    set vvar(x) [lindex $var $i]
	    incr i
	    set vvar(y) [lindex $var $i]

	    IMGSVRApply $vvarname 1
	}
	x {
	    incr i
	    set vvar(x) [lindex $var $i]
	}
	y {
	    incr i
	    set vvar(y) [lindex $var $i]
	}
	name {
	    if {$flag && $base} {
		MultiLoad fc
	    }
	    incr i
	    set vvar(name) [lindex $var $i]

	    IMGSVRApply $vvarname 1
	}
	default {
	    if {$flag && $base} {
		MultiLoad fc
	    }
	    set vvar(name) [lindex $var $i]

	    IMGSVRApply $vvarname 1
	}
    }
}

