#  Copyright (C) 1999-2007
#  Smithsonian Astrophysical Observatory, Cambridge, MA, USA
#  For conditions of distribution and use, see copyright notice in "copyright"

package provide DS9 1.0

proc DSSSAODialog {} {
    global dsssao

    if [winfo exists $dsssao(top)] {
	raise $dsssao(top)
	return
    }

    set varname dsssaoi
    upvar #0 $varname var
    global $varname

    set var(top) $dsssao(top)
    set var(mb) $dsssao(mb)
    set var(system) $dsssao(system)
    set var(sky) $dsssao(sky)
    set var(skyformat) $dsssao(skyformat)
    set var(rformat) $dsssao(rformat)
    set var(width) $dsssao(width)
    set var(height) $dsssao(height)
    set var(mode) $dsssao(mode)
    set var(save) $dsssao(save)
    set var(valid) $dsssao(valid)

    set w $var(top)
    IMGSVRInit $varname "DSS-SAO [msgcat::mc {Server}]" DSSSAOExec DSSSAOAck

    # menu
    IMGSVRFileMenu $varname
    AREditMenu $varname
    NSVRServerMenu $varname
    ARCoordMenu $varname
    ARSizeMenu $varname

    IMGSVRPrefsMenu $varname
    IMGSVRUpdate $varname 1
}

proc DSSSAOExec {varname} {
    upvar #0 $varname var
    global $varname

    if {$var(save)} {
	set compress no
	set var(fn) [SaveFileDialog dsssaofbox]
	if {$var(fn) == {}} {
	    return
	}

    } else {
	set compress gzip
	set var(fn) [tmpnam dss ".fits.gz"]
    }

    # skyformat
    switch -- $var(skyformat) {
	degrees {
	    set xx [uformat d h: $var(x)]
	    set yy [uformat d d: $var(y)]
	}
	sexagesimal {
	    set xx $var(x)
	    set yy $var(y)
	}
    }

    # size - change to arcmin
    switch -- $var(rformat) {
	degrees {
	    set ww [expr $var(width)*60.]
	    set hh [expr $var(height)*60.]
	}
	arcmin {
	    set ww $var(width)
	    set hh $var(height)
	}
	arcsec {
	    set ww [expr $var(width)/60.]
	    set hh [expr $var(height)/60.]
	}
    }
    if {$ww>60} {
	set ww 60
    }
    if {$hh>60} {
	set hh 60
    }

    # query
    set var(query) [http::formatQuery r $xx d $yy e J2000 \
			w $ww h $ww \
			c $compress]
    set var(url) "http://archive.harvard.edu/archive/dss"
    IMGSVRLoad $varname
}

proc DSSSAOAck {varname} {
    upvar #0 $varname var
    global $varname

set msg {Acknowledgments for the DSS-SAO

The Digitized Sky Surveys were produced at the Space Telescope Science
Institute under U.S.  Government grant NAG W-2166. The images of these
surveys are based on photographic data obtained using the Oschin
Schmidt Telescope on Palomar Mountain and the UK Schmidt Telescope.
The plates were processed into the present compressed digital form
with the permission of these institutions.

The Oschin Schmidt Telescope is operated by the California Institute
of Technology and Palomar Observatory.

The UK Schmidt Telescope was operated by the Royal Observatory
Edinburgh, with funding from the UK Science and Engineering Research
Council (later the UK Particle Physics and Astronomy Research
Council), until 1988 June, and thereafter by the Anglo-Australian
Observatory. The blue plates of the southern Sky Atlas and its
Equatorial Extension (together known as the SERC-J), as well as the
Equatorial Red (ER), and the Second Epoch [red] Survey (SES) were all
taken with the UK Schmidt.
}

    SimpleTextDialog dsssaoack Acknowledgment 80 40 insert top $msg
}

# Process Cmds

proc ProcessDSSSAOCmd {varname iname flag fcName} {
    upvar $varname var
    upvar $iname i
    upvar $fcName fc

    global current

    set vvarname dsssaoi
    upvar #0 $vvarname vvar
    global $vvarname

    # do we have a rgb frame
    if {[$current(frame) get type] == {rgb}} {
	set base 0
    } else {
	set base 1
    }

    DSSSAODialog

    set vvar(mode) current

    switch -- [string tolower [lindex $var $i]] {
	size {
	    incr i
	    set vvar(width) [lindex $var $i]
	    incr i
	    set vvar(height) [lindex $var $i]
	}
	width {
	    incr i
	    set vvar(width) [lindex $var $i]
	}
	height {
	    incr i
	    set vvar(height) [lindex $var $i]
	}
	coord {
	    if {$flag} {
		MultiLoad fc
	    }
	    incr i
	    set vvar(x) [lindex $var $i]
	    incr i
	    set vvar(y) [lindex $var $i]

	    IMGSVRApply $vvarname 1
	}
	x {
	    incr i
	    set vvar(x) [lindex $var $i]
	}
	y {
	    incr i
	    set vvar(y) [lindex $var $i]
	}
	name {
	    if {$flag && $base} {
		MultiLoad fc
	    }
	    incr i
	    set vvar(name) [lindex $var $i]

	    IMGSVRApply $vvarname 1
	}
	default {
	    if {$flag && $base} {
		MultiLoad fc
	    }
	    set vvar(name) [lindex $var $i]

	    IMGSVRApply $vvarname 1
	}
    }
}

