#  Copyright (C) 1999-2007
#  Smithsonian Astrophysical Observatory, Cambridge, MA, USA
#  For conditions of distribution and use, see copyright notice in "copyright"

package provide DS9 1.0

proc EditMenu {} {
    global ds9

    menu $ds9(mb).edit
    $ds9(mb).edit add command -label [msgcat::mc {Undo}] -command UndoFrame
    $ds9(mb).edit add separator
    $ds9(mb).edit add command -label [msgcat::mc {Cut}] -command CutFrame
    $ds9(mb).edit add command -label [msgcat::mc {Copy}] -command CopyFrame
    $ds9(mb).edit add command -label [msgcat::mc {Paste}] -command PasteFrame
    $ds9(mb).edit add separator
    $ds9(mb).edit add radiobutton -label [msgcat::mc {None}] \
	-variable ds9(mode) -value none -command ChangeMode
    $ds9(mb).edit add radiobutton -label [msgcat::mc {Pointer}] \
	-variable ds9(mode) -value pointer -command ChangeMode
    $ds9(mb).edit add radiobutton -label [msgcat::mc {Crosshair}] \
	-variable ds9(mode) -value crosshair -command ChangeMode
    $ds9(mb).edit add radiobutton -label [msgcat::mc {Colorbar}] \
	-variable ds9(mode) -value colorbar -command ChangeMode
    $ds9(mb).edit add radiobutton -label [msgcat::mc {Pan}] \
	-variable ds9(mode) -value pan -command ChangeMode
    $ds9(mb).edit add radiobutton -label [msgcat::mc {Zoom}] \
	-variable ds9(mode) -value zoom -command ChangeMode
    $ds9(mb).edit add radiobutton -label [msgcat::mc {Rotate}] \
	-variable ds9(mode) -value rotate -command ChangeMode
    $ds9(mb).edit add radiobutton -label [msgcat::mc {Catalog}] \
	-variable ds9(mode) -value catalog -command ChangeMode
    $ds9(mb).edit add radiobutton -label [msgcat::mc {Examine}] \
	-variable ds9(mode) -value examine -command ChangeMode
    switch [tk windowingsystem] {
	x11 -
	win32 {
	    $ds9(mb).edit add separator
	    $ds9(mb).edit add cascade -menu $ds9(mb).prefs \
		-label "[msgcat::mc {Preferences}]..." 
#	    $ds9(mb).edit add command -label "[msgcat::mc {Preferences}]..." 
#		-command PrefsDialog
	}
	aqua {proc ::tk::mac::ShowPreferences {} {PrefsDialog}}
    }

    EditPrefsMenu
}

proc EditPrefsMenu {} {
    global ds9
    global prefs

    menu $ds9(mb).prefs.gui.edit
    $ds9(mb).prefs.gui.edit add radiobutton -label [msgcat::mc {None}] \
	-variable prefs(ds9,mode) -value none -command SavePrefs
    $ds9(mb).prefs.gui.edit add radiobutton -label [msgcat::mc {Pointer}] \
	-variable prefs(ds9,mode) -value pointer -command SavePrefs
    $ds9(mb).prefs.gui.edit add radiobutton -label [msgcat::mc {Crosshair}] \
	-variable prefs(ds9,mode) -value crosshair -command SavePrefs
    $ds9(mb).prefs.gui.edit add radiobutton -label [msgcat::mc {Colorbar}] \
	-variable prefs(ds9,mode) -value colorbar -command SavePrefs
    $ds9(mb).prefs.gui.edit add radiobutton -label [msgcat::mc {Pan}] \
	-variable prefs(ds9,mode) -value pan -command SavePrefs
    $ds9(mb).prefs.gui.edit add radiobutton -label [msgcat::mc {Zoom}] \
	-variable prefs(ds9,mode) -value zoom -command SavePrefs
    $ds9(mb).prefs.gui.edit add radiobutton -label [msgcat::mc {Rotate}] \
	-variable prefs(ds9,mode) -value rotate -command SavePrefs
    $ds9(mb).prefs.gui.edit add radiobutton -label [msgcat::mc {Catalog}] \
	-variable prefs(ds9,mode) -value catalog -command SavePrefs
    $ds9(mb).prefs.gui.edit add radiobutton -label [msgcat::mc {Examine}] \
	-variable prefs(ds9,mode) -value examine -command SavePrefs
}

proc EditButtons {} {
    global ds9

    frame $ds9(buttons).edit
    ButtonButton $ds9(buttons).edit.undo \
	[string tolower [msgcat::mc {Undo}]] UndoFrame
    ButtonButton $ds9(buttons).edit.cut \
	[string tolower [msgcat::mc {Cut}]] CutFrame
    ButtonButton $ds9(buttons).edit.copy \
	[string tolower [msgcat::mc {Copy}]] CopyFrame
    ButtonButton $ds9(buttons).edit.paste \
	[string tolower [msgcat::mc {Paste}]] PasteFrame
    RadioButton $ds9(buttons).edit.default \
	[string tolower [msgcat::mc {None}]] \
	ds9(mode) none ChangeMode
    RadioButton $ds9(buttons).edit.pointer \
	[string tolower [msgcat::mc {Pointer}]] \
	ds9(mode) pointer ChangeMode
    RadioButton $ds9(buttons).edit.crosshair \
	[string tolower [msgcat::mc {Crosshair}]] \
	ds9(mode) crosshair ChangeMode
    RadioButton $ds9(buttons).edit.colorbar \
	[string tolower [msgcat::mc {Colorbar}]] \
	ds9(mode) colorbar ChangeMode
    RadioButton $ds9(buttons).edit.pan \
	[string tolower [msgcat::mc {Pan}]] \
	ds9(mode) pan ChangeMode
    RadioButton $ds9(buttons).edit.zoom \
	[string tolower [msgcat::mc {Zoom}]] \
	ds9(mode) zoom ChangeMode
    RadioButton $ds9(buttons).edit.rotate \
	[string tolower [msgcat::mc {Rotate}]] \
	ds9(mode) rotate ChangeMode
    RadioButton $ds9(buttons).edit.catalog \
	[string tolower [msgcat::mc {Catalog}]] \
	ds9(mode) catalog ChangeMode
    RadioButton $ds9(buttons).edit.examine \
	[string tolower [msgcat::mc {Examine}]] \
	ds9(mode) examine ChangeMode
}

proc UpdateEditMenu {} {
    global ds9
    global current
    global marker

    global debug
    if {$debug(tcl,update)} {
	puts "UpdateEditMenu"
    }

    switch -- $ds9(mode) {
	pointer {
	    if {$current(frame) != {}} {
		set l [$current(frame) has marker undo]
		if {$l != {}} {
		    $ds9(mb).edit entryconfig [msgcat::mc {Undo}] \
			-state normal
		    $ds9(buttons).edit.undo configure -state normal
		} else {
		    $ds9(mb).edit entryconfig [msgcat::mc {Undo}] \
			-state disabled
		    $ds9(buttons).edit.undo configure -state disabled
		}

		if {[$current(frame) has marker select]} {
		    $ds9(mb).edit entryconfig [msgcat::mc {Cut}] -state normal
		    $ds9(mb).edit entryconfig [msgcat::mc {Copy}] -state normal
		    $ds9(buttons).edit.cut configure -state normal
		    $ds9(buttons).edit.copy configure -state normal
		} else {
		    $ds9(mb).edit entryconfig [msgcat::mc {Cut}] \
			-state disabled
		    $ds9(mb).edit entryconfig [msgcat::mc {Copy}] \
			-state disabled
		    $ds9(buttons).edit.cut configure -state disabled
		    $ds9(buttons).edit.copy configure -state disabled
		}

		if {$marker(copy) != {} } {
		    if {[$marker(copy) has marker paste]} {
			$ds9(mb).edit entryconfig [msgcat::mc {Paste}] \
			    -state normal
			$ds9(buttons).edit.paste configure -state normal
		    } else {
			$ds9(mb).edit entryconfig [msgcat::mc {Paste}] \
			    -state disabled
			$ds9(buttons).edit.paste configure -state disabled
		    }
		} else {
		    $ds9(mb).edit entryconfig [msgcat::mc {Paste}] \
			-state disabled
		    $ds9(buttons).edit.paste configure -state disabled
		}
	    } else {
		$ds9(mb).edit entryconfig [msgcat::mc {Undo}] -state disabled
		$ds9(mb).edit entryconfig [msgcat::mc {Cut}] -state disabled
		$ds9(mb).edit entryconfig [msgcat::mc {Copy}] -state disabled
		$ds9(mb).edit entryconfig [msgcat::mc {Paste}] -state disabled

		$ds9(buttons).edit.undo configure -state disabled
		$ds9(buttons).edit.cut configure -state disabled
		$ds9(buttons).edit.copy configure -state disabled
		$ds9(buttons).edit.paste configure -state disabled
	    }
	}
	none -
	crosshair -
	colorbar -
	pan -
	zoom -
	rotate -
	examine -
	illustrate -
	catalog -
	imexam {$ds9(mb).edit entryconfig [msgcat::mc {Undo}] -state disabled}
    }
}

