#  Copyright (C) 1999-2007
#  Smithsonian Astrophysical Observatory, Cambridge, MA, USA
#  For conditions of distribution and use, see copyright notice in "copyright"

package provide DS9 1.0

proc CreateMenuBar {} {
    global ds9

    menu $ds9(mb) 
    switch [tk windowingsystem] {
	x11 -
	win32 {}
	aqua {$ds9(mb) add cascade -menu $ds9(mb).apple}
    }
    $ds9(mb) add cascade -label [msgcat::mc {File}] -menu $ds9(mb).file
    $ds9(mb) add cascade -label [msgcat::mc {Edit}] -menu $ds9(mb).edit
    $ds9(mb) add cascade -label [msgcat::mc {View}] -menu $ds9(mb).view
    $ds9(mb) add cascade -label [msgcat::mc {Frame}] -menu $ds9(mb).frame
    $ds9(mb) add cascade -label [msgcat::mc {Bin}] -menu $ds9(mb).bin
    $ds9(mb) add cascade -label [msgcat::mc {Zoom}] -menu $ds9(mb).zoom
    $ds9(mb) add cascade -label [msgcat::mc {Scale}] -menu $ds9(mb).scale
    $ds9(mb) add cascade -label [msgcat::mc {Color}] -menu $ds9(mb).color
    $ds9(mb) add cascade -label [msgcat::mc {Region}] -menu $ds9(mb).region
    $ds9(mb) add cascade -label [msgcat::mc {WCS}] -menu $ds9(mb).wcs
    $ds9(mb) add cascade -label [msgcat::mc {Analysis}] -menu $ds9(mb).analysis
    $ds9(mb) add cascade -label [msgcat::mc {Help}] -menu $ds9(mb).help

    PrefsMenu

    FileMenu
    EditMenu
    ViewMenu
    FrameMenu
    BinMenu
    ZoomMenu
    ScaleMenu
    ColorMenu
    RegionMenu
    WCSMenu
    AnalysisMenu
    HelpMenu
}

# full coordinate menu
proc UpdateCoordMenu {frame which} {
    global current
    global ds9

    # no frame
    if {$frame == {}} {
	SetCoordMenu $which normal
	return
    }

    # unloaded frame
    if {![$frame has fits]} {
	SetCoordMenu $which normal
	return
    }

    # loaded frame
    if {[$frame has wcs WCS]} {
	$which entryconfig [msgcat::mc {WCS}] -state normal
    } else {
	$which entryconfig [msgcat::mc {WCS}] -state disabled
    }
    $which entryconfig [msgcat::mc {Multiple WCS}] -state normal
    foreach l {a b c d e f g h i j k l m n o p q r s t u v w x y z} {
	if {[$frame has wcs WCS$l]} {
	    $which.wcs entryconfig "[msgcat::mc {WCS}] $l" -state normal
	} else {
	    $which.wcs entryconfig "[msgcat::mc {WCS}] $l" -state disabled
	}
    }

    if {[$frame has wcs equatorial WCS]} {
	SetSkyFrameMenu $which normal
    } else {
	SetSkyFrameMenu $which disabled
    }

    if {[$frame has detector]} {
	$which entryconfig [msgcat::mc {Detector}] -state normal
    } else {
	$which entryconfig [msgcat::mc {Detector}] -state disabled
    }
    
    if {[$frame has amplifier]} {
	$which entryconfig [msgcat::mc {Amplifier}] -state normal
    } else {
	$which entryconfig [msgcat::mc {Amplifier}] -state disabled
    }

    if {[$frame has physical]} {
	$which entryconfig [msgcat::mc {Physical}] -state normal
    } else {
	$which entryconfig [msgcat::mc {Physical}] -state disabled
    }

    $which entryconfig [msgcat::mc {Image}] -state normal
}

proc SetCoordMenu {which value} {
    global ds9

    $which entryconfig [msgcat::mc {WCS}] -state $value
    $which entryconfig [msgcat::mc {Multiple WCS}] -state $value
    $which entryconfig [msgcat::mc {Image}] -state $value
    $which entryconfig [msgcat::mc {Physical}] -state $value
    $which entryconfig [msgcat::mc {Amplifier}] -state $value
    $which entryconfig [msgcat::mc {Detector}] -state $value

    $which entryconfig "[msgcat::mc {Equatorial}] B1950" -state $value
    $which entryconfig "[msgcat::mc {Equatorial}] J2000" -state $value
    $which entryconfig [msgcat::mc {ICRS}] -state $value
    $which entryconfig [msgcat::mc {Galactic}] -state $value
    $which entryconfig [msgcat::mc {Ecliptic}] -state $value

    $which entryconfig [msgcat::mc {Degrees}] -state $value
    $which entryconfig {Sexagesimal} -state $value

    foreach l {a b c d e f g h i j k l m n o p q r s t u v w x y z} {
	$which.wcs entryconfig "[msgcat::mc {WCS}] $l" -state $value
    }
}

proc SetSkyFrameMenu {which value} {
    $which entryconfig "[msgcat::mc {Equatorial}] B1950" -state $value
    $which entryconfig "[msgcat::mc {Equatorial}] J2000" -state $value
    $which entryconfig [msgcat::mc {ICRS}] -state $value
    $which entryconfig [msgcat::mc {Galactic}] -state $value
    $which entryconfig [msgcat::mc {Ecliptic}] -state $value

    $which entryconfig [msgcat::mc {Degrees}] -state $value
    $which entryconfig {Sexagesimal} -state $value
}

proc SetSkyFrameMenu2 {which value} {
    $which entryconfig "[msgcat::mc {Equatorial}] B1950" -state $value
    $which entryconfig "[msgcat::mc {Equatorial}] J2000" -state $value
    $which entryconfig [msgcat::mc {ICRS}] -state $value
    $which entryconfig [msgcat::mc {Galactic}] -state $value
    $which entryconfig [msgcat::mc {Ecliptic}] -state $value
}

# dist menu
proc UpdateDistMenu {frame which defnoframe defnofits} {
    global current
    global ds9

    # no frame
    if {$frame == {}} {
	if {$defnoframe} {
	    SetDistMenu $which normal
	} else {
	    SetDistMenu $which disabled
	}
	return
    }

    # unloaded frame
    if {![$frame has fits]} {
	if {$defnofits} {
	    SetDistMenu $which normal
	} else {
	    SetDistMenu $which disabled
	}
	return
    }

    # loaded frame
    if {[$frame has wcs WCS]} {
	$which entryconfig [msgcat::mc {WCS}] -state normal
    } else {
	$which entryconfig [msgcat::mc {WCS}] -state disabled
    }
    $which entryconfig [msgcat::mc {Multiple WCS}] -state normal
    foreach l {a b c d e f g h i j k l m n o p q r s t u v w x y z} {
	if {[$frame has wcs WCS$l]} {
	    $which.wcs entryconfig "[msgcat::mc {WCS}] $l" -state normal
	} else {
	    $which.wcs entryconfig "[msgcat::mc {WCS}] $l" -state disabled
	}
    }

    if {[$frame has wcs equatorial WCS]} {
	SetSkyFormatMenu $which normal
    } else {
	SetSkyFormatMenu $which disabled
    }

    if {[$frame has detector]} {
	$which entryconfig [msgcat::mc {Detector}] -state normal
    } else {
	$which entryconfig [msgcat::mc {Detector}] -state disabled
    }

    if {[$frame has amplifier]} {
	$which entryconfig [msgcat::mc {Amplifier}] -state normal
    } else {
	$which entryconfig [msgcat::mc {Amplifier}] -state disabled
    }

    if {[$frame has physical]} {
	$which entryconfig [msgcat::mc {Physical}] -state normal
    } else {
	$which entryconfig [msgcat::mc {Physical}] -state disabled
    }

    $which entryconfig [msgcat::mc {Image}] -state normal
}

proc SetDistMenu {which value} {
    global ds9

    $which entryconfig [msgcat::mc {WCS}] -state $value
    $which entryconfig [msgcat::mc {Multiple WCS}] -state $value
    $which entryconfig [msgcat::mc {Image}] -state $value
    $which entryconfig [msgcat::mc {Physical}] -state $value
    $which entryconfig [msgcat::mc {Amplifier}] -state $value
    $which entryconfig [msgcat::mc {Detector}] -state $value

    $which entryconfig [msgcat::mc {Degrees}] -state $value
    $which entryconfig [msgcat::mc {ArcMin}] -state $value
    $which entryconfig [msgcat::mc {ArcSec}] -state $value

    foreach l {a b c d e f g h i j k l m n o p q r s t u v w x y z} {
	$which.wcs entryconfig "[msgcat::mc {WCS}] $l" -state $value
    }
}

proc SetSkyFormatMenu {which value} {
    $which entryconfig [msgcat::mc {Degrees}] -state $value
    $which entryconfig [msgcat::mc {ArcMin}] -state $value
    $which entryconfig [msgcat::mc {ArcSec}] -state $value
}

proc AdjustCoord {frame sysname} {
    upvar $sysname sys

    if {$frame != {}} {
	if {[$frame has fits]} {
	    switch -- $sys {
		image {}
		physical {
		    if {![$frame has physical]} {
			set sys image
		    }
		}
		amplifier {
		    if {![$frame has amplifier]} {
			set sys image
		    }
		}
		detector {
		    if {![$frame has detector]} {
			set sys image
		    }
		}
		wcs {
		    if {![$frame has wcs $sys]} {
			set sys image
		    }
		}
		default {
		    if {![$frame has wcs $sys]} {
			if [$frame has wcs wcs] {
			    set sys wcs
			} else {
			    set sys image
			}
		    }
		}
	    }
	}
    }
}

proc AdjustDist {frame sysname} {
    upvar $sysname sys
    global current

    if {$frame != {}} {
	if {[$frame has fits]} {
	    switch -- $sys {
		image {}
		physical {
		    if {![$frame has physical]} {
			set sys image
		    }
		}
		amplifier {
		    if {![$frame has amplifier]} {
			set sys image
		    }
		}
		detector {
		    if {![$frame has detector]} {
			set sys image
		    }
		}
		default {
		    if {![$frame has wcs $sys]} {
			set sys wcs
		    }
		}
	    }
	}
    }
}

proc OptionMenu {w varName args} {
    upvar #0 $varName var
    global $varName

    menubutton $w -indicatoron 1 -menu $w.menu -relief raised -bd 2 \
	-highlightthickness 2 -anchor c -direction flush
    menu $w.menu -tearoff 0
    foreach ff $args {
	set aa [lindex $ff 0]
	set bb [lindex $ff 1]
	if {$bb == $var} {
	    $w config -text $aa
	}

    	$w.menu add radiobutton -label $aa \
	    -variable $varName -value $bb \
	    -command [list $w config -text $aa]
    }

    return $w.menu
}
