#  Copyright (C) 1999-2007
#  Smithsonian Astrophysical Observatory, Cambridge, MA, USA
#  For conditions of distribution and use, see copyright notice in "copyright"

package provide DS9 1.0

proc UpdatePTDialog {which x y sys} {
    global pixel
    global pixelValues

    if {$pixel(view)} {
	$which get pixel table $sys $x $y $pixel(size) pixelValues
    }
}

proc PTDialog {} {
    global ds9
    global pixel

    set pixel(view) 1
    if {[winfo exists $pixel(top)]} {
	raise $pixel(top)
	PTConfigure
	return
    }
	
    # create the pixel table window

    set w $pixel(top)
    set title [msgcat::mc {Pixel Table}]

    toplevel $w
    wm title $w $title
    wm iconname $w $title
    wm group $w $ds9(top)
    wm protocol $w WM_DELETE_WINDOW PTDestroyDialog
    wm transient $w $ds9(top)

    $w configure -menu $pixel(mb)

    # simple menus

    menu $pixel(mb)
    $pixel(mb) add cascade -label [msgcat::mc {File}] -menu $pixel(mb).file
    $pixel(mb) add cascade -label [msgcat::mc {Edit}] -menu $pixel(mb).edit
    $pixel(mb) add cascade -label Size -menu $pixel(mb).size

    menu $pixel(mb).file
    $pixel(mb).file add command -label "[msgcat::mc {Save}]..." \
	-command PTSaveDialog
    $pixel(mb).file add separator
    $pixel(mb).file add command -label [msgcat::mc {Close}] \
	-command PTDestroyDialog

    menu $pixel(mb).edit
    $pixel(mb).edit add command -label [msgcat::mc {Cut}] \
	-state disabled
    $pixel(mb).edit add command -label [msgcat::mc {Copy}] \
	-command PTCopyDialog
    $pixel(mb).edit add command -label [msgcat::mc {Paste}] \
	-state disabled
    $pixel(mb).edit add command -label [msgcat::mc {Clear}] \
	-command PTClearDialog

    menu $pixel(mb).size
    $pixel(mb).size add radiobutton -label {3x3} -variable pixel(size) \
	-value 3 -command PTConfigure
    $pixel(mb).size add radiobutton -label {5x5} -variable pixel(size) \
	-value 5 -command PTConfigure
    $pixel(mb).size add radiobutton -label {7x7} -variable pixel(size) \
	-value 7 -command PTConfigure
    $pixel(mb).size add radiobutton -label {9x9} -variable pixel(size) \
	-value 9 -command PTConfigure

    frame $w.table
    pack $w.table

    selection handle $pixel(top) PTExportSelection

    PTConfigure
}

proc PTDestroyDialog {} {
    global pixel
    global pixelValues
    
    set pixel(view) 0
    if {[winfo exists $pixel(top)]} {
	destroy $pixel(top)
	destroy $pixel(mb)
    }

    if {[info exists pixelValues]} {
	unset pixelValues
    }
}

proc PTCopyDialog {} {
    global pixel

    set pixel(copy) {}
    PTRender pixel(copy)

    selection own -command PTLostSelection $pixel(top)

    clipboard clear
    clipboard append $pixel(copy)
}

proc PTClearDialog {} {
    global pixel
    global pixelValues

    if {$pixel(view)} {
	for {set i 0} {$i<=$pixel(size)} {incr i} {
	    for {set j 0} {$j<=$pixel(size)} {incr j} {
		set pixelValues($i$j) {}
	    }
	}
    }
}

proc PTSaveDialog {} {
    set filename [SaveFileDialog pixelfbox]

    if {$filename != {}} {
	set file [open $filename w]

	set cp {}
	PTRender cp
	puts -nonewline $file $cp
	
	close $file
    }
}

# support

proc PTConfigure {} {
    global pixel
    global pixelValues

    set w $pixel(top)

    # ungrid all
    for {set j 0} {$j<=$pixel(maxsize)} {incr j} {
	for {set i 0} {$i<=$pixel(maxsize)} {incr i} {
	    if {[winfo exists $w.table.b$i$j]} {
		grid forget $w.table.b$i$j
	    }
	}
    }
    if {[info exists pixelValues]} {
	unset pixelValues
    }

    # corner
    set i 0
    set j 0
    if {![winfo exists $w.table.b$i$j]} {
	label $w.table.b$i$j -width 12 -relief groove -borderwidth 1
    }
    grid $w.table.b$i$j -column $i -row $j

    # x (column)
    set j 0
    for {set i 1} {$i<=$pixel(size)} {incr i} {
	if {![winfo exists $w.table.b$i$j]} {
	    label $w.table.b$i$j -width 12 -relief groove -borderwidth 1 \
		-textvariable pixelValues($i$j) -foreground blue
	}
	grid $w.table.b$i$j -column $i -row $j
    }

    # y (row)
    set i 0
    for {set j 1} {$j<=$pixel(size)} {incr j} {
	if {![winfo exists $w.table.b$i$j]} {
	    label $w.table.b$i$j -width 12 -relief groove -borderwidth 1 \
		-textvariable pixelValues($i$j) -foreground blue
	}
	grid $w.table.b$i$j -column $i -row [expr $pixel(size)-$j+1] 
    }

    # body
    set half [expr ($pixel(size)+1)/2]
    set fg [$w.table.b00 cget -foreground]

    for {set i 1} {$i<=$pixel(size)} {incr i} {
	for {set j 1} {$j<=$pixel(size)} {incr j} {
	    if {![winfo exists $w.table.b$i$j]} {
		label $w.table.b$i$j -width 12 -relief groove -borderwidth 1 \
		    -textvariable pixelValues($i$j)
	    }
	    if {($i==$half) && ($j==$half)} {
		$w.table.b$i$j configure -foreground red
	    } else {
		$w.table.b$i$j configure -foreground $fg
	    }
	    grid $w.table.b$i$j -column $i -row [expr $pixel(size)-$j+1]
	}
    }

    # show
    pack $w.table
}

proc PTRender {varname} {
    upvar $varname var
    global pixel
    global pixelValues

    # col header
    append var "            "
    for {set i 1} {$i<=$pixel(size)} {incr i} {
	set msg [format "%12s" $pixelValues(${i}0)]
	append var "$msg"
    }
    append var "\n"

    append var "            "
    for {set i 1} {$i<=$pixel(size)} {incr i} {
	append var " -----------"
    }
    append var "\n"

    # body
    for {set j $pixel(size)} {$j>=1} {incr j -1} {
	set msg [format "%10s" $pixelValues(0${j})]
	append var "$msg |"
	for {set i 1} {$i<=$pixel(size)} {incr i} {
	    set msg [format "%12.11s" $pixelValues($i$j)]
	    append var "$msg"
	}
	append var "\n"
    }
}

proc PTExportSelection {offset bytes} {
    global pixel

    if {$pixel(copy) != {}} {
	return [string range $pixel(copy) $offset [expr $offset+$bytes]]
    }
}

proc PTLostSelection {} {
    global pixel

    set pixel(copy) {}
}

# Process Cmds

proc ProcessPixelTableCmd {varname iname} {
    upvar $varname var
    upvar $iname i

    if {[FromYesNo [lindex $var $i]]} {
	PTDialog
    } else {
	PTDestroyDialog
    }
}
